@@ -1,6 +1,6 @@
 # -*- Mode:Python; indent-tabs-mode:nil; tab-width:4 -*-
 #
-# Copyright (C) 2015, 2016 Canonical Ltd
+# Copyright (C) 2015-2017 Canonical Ltd
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License version 3 as
@@ -16,15 +16,19 @@
 
 import fileinput
 import os
-import shutil
+import re
 import subprocess
 import time
 import uuid
+import xdg
+from distutils import dir_util
 
 import fixtures
 import pexpect
+from unittest import mock
 import testtools
 from testtools import content
+from testtools.matchers import MatchesRegex
 
 from snapcraft.tests import fixture_setup
 
@@ -49,39 +53,114 @@ class TestCase(testtools.TestCase):
 
         self.useFixture(fixtures.EnvironmentVariable(
             'XDG_CONFIG_HOME', os.path.join(self.path, '.config')))
+        self.useFixture(fixtures.EnvironmentVariable(
+            'XDG_CACHE_HOME', os.path.join(self.path, '.cache')))
         self.useFixture(fixtures.EnvironmentVariable(
             'XDG_DATA_HOME', os.path.join(self.path, 'data')))
+        self.useFixture(fixtures.EnvironmentVariable('TERM', 'dumb'))
+
+        patcher = mock.patch(
+            'xdg.BaseDirectory.xdg_config_home',
+            new=os.path.join(self.path, '.config'))
+        patcher.start()
+        self.addCleanup(patcher.stop)
+        patcher = mock.patch(
+            'xdg.BaseDirectory.xdg_data_home',
+            new=os.path.join(self.path, 'data'))
+        patcher.start()
+        self.addCleanup(patcher.stop)
+        patcher = mock.patch(
+            'xdg.BaseDirectory.xdg_cache_home',
+            new=os.path.join(self.path, '.cache'))
+        patcher.start()
+        self.addCleanup(patcher.stop)
+
+        patcher_dirs = mock.patch(
+            'xdg.BaseDirectory.xdg_config_dirs',
+            new=[xdg.BaseDirectory.xdg_config_home])
+        patcher_dirs.start()
+        self.addCleanup(patcher_dirs.stop)
+
+        patcher_dirs = mock.patch(
+            'xdg.BaseDirectory.xdg_data_dirs',
+            new=[xdg.BaseDirectory.xdg_data_home])
+        patcher_dirs.start()
+        self.addCleanup(patcher_dirs.stop)
+
+        # Note that these directories won't exist when the test starts,
+        # they might be created after calling the snapcraft command on the
+        # project dir.
+        self.parts_dir = 'parts'
+        self.stage_dir = 'stage'
+        self.prime_dir = 'prime'
+
+    def run_snapcraft(
+            self, command, project_dir=None, debug=True,
+            pre_func=lambda: None):
+        if project_dir:
+            self.copy_project_to_cwd(project_dir)
 
-    def run_snapcraft(self, command, project_dir=None, yaml_dir=None):
         if isinstance(command, str):
             command = [command]
-        if project_dir:
-            if not os.path.exists(project_dir):
-                cwd = self.copy_project_to_tmp(project_dir)
-            else:
-                cwd = os.path.join(self.path, project_dir)
-        else:
-            cwd = None
-
-        if yaml_dir:
-            cwd = os.path.join(self.path, yaml_dir)
-
+        snapcraft_command = [self.snapcraft_command]
+        if debug:
+            snapcraft_command.append('-d')
         try:
-            return subprocess.check_output(
-                [self.snapcraft_command, '-d'] + command, cwd=cwd,
+            pre_func()
+            snapcraft_output = subprocess.check_output(
+                snapcraft_command + command,
                 stderr=subprocess.STDOUT, universal_newlines=True)
         except subprocess.CalledProcessError as e:
             self.addDetail('output', content.text_content(e.output))
             raise
 
-    def copy_project_to_tmp(self, project_dir):
-        tmp_project_dir = os.path.join(self.path, project_dir)
-        shutil.copytree(
-            os.path.join(self.snaps_dir, project_dir), tmp_project_dir,
-            symlinks=True)
-        return tmp_project_dir
+        if not os.getenv('SNAPCRAFT_IGNORE_APT_AUTOREMOVE', False):
+            self.addCleanup(self.run_apt_autoremove)
 
-    def get_output_ignoring_non_zero_exit(self, binary, cwd):
+        return snapcraft_output
+
+    def run_snapcraft_parser(self, arguments):
+        try:
+            snapcraft_output = subprocess.check_output(
+                [self.snapcraft_parser_command, '-d'] + arguments,
+                stderr=subprocess.STDOUT, universal_newlines=True)
+        except subprocess.CalledProcessError as e:
+            self.addDetail('output', content.text_content(e.output))
+            raise
+        return snapcraft_output
+
+    def run_apt_autoremove(self):
+        deb_env = os.environ.copy()
+        deb_env.update({
+            'DEBIAN_FRONTEND': 'noninteractive',
+            'DEBCONF_NONINTERACTIVE_SEEN': 'true',
+        })
+
+        try:
+            autoremove_output = subprocess.check_output(
+                'sudo apt-get autoremove -y'.split(),
+                stderr=subprocess.STDOUT, env=deb_env)
+            self.addDetail(
+                'apt-get autoremove output',
+                content.text_content(autoremove_output.decode('utf-8')))
+        except subprocess.CalledProcessError as e:
+            self.addDetail(
+                'apt-get autoremove error', content.text_content(str(e)))
+            self.addDetail(
+                'apt-get autoremove output',
+                content.text_content(e.output.decode('utf-8')))
+
+            if os.getenv('SNAPCRAFT_APT_AUTOREMOVE_CHECK_FAIL', False):
+                raise
+
+    def copy_project_to_cwd(self, project_dir):
+        # Because cwd already exists, shutil.copytree would raise
+        # FileExistsError. Use the lesser known distutils.dir_util.copy_tree
+        dir_util.copy_tree(
+            os.path.join(self.snaps_dir, project_dir), self.path,
+            preserve_symlinks=True)
+
+    def get_output_ignoring_non_zero_exit(self, binary, cwd=None):
         # Executing the binaries exists > 0 on trusty.
         # TODO investigate more to understand the cause.
         try:
@@ -119,12 +198,15 @@ class StoreTestCase(TestCase):
 
     def logout(self):
         output = self.run_snapcraft('logout')
-        expected = ('Clearing credentials for Ubuntu One SSO.\n'
-                    'Credentials cleared.\n')
-        self.assertEqual(expected, output)
+        expected = (r'.*Clearing credentials for Ubuntu One SSO.\n.*\n'
+                    r'Credentials cleared.\n.*')
+        self.assertThat(output, MatchesRegex(expected, flags=re.DOTALL))
 
-    def register(self, snap_name, wait=True):
-        self.run_snapcraft(['register', snap_name])
+    def register(self, snap_name, private=False, wait=True):
+        command = ['register', snap_name]
+        if private:
+            command.append('--private')
+        self.run_snapcraft(command)
         # sleep a few seconds to avoid hitting the store restriction on
         # following registrations.
         if wait:
@@ -169,29 +251,59 @@ class StoreTestCase(TestCase):
         process.close()
         return process.exitstatus
 
-    def update_name_and_version(self, project_dir, name=None, version=None):
+    def list_registered(self, expected_snaps):
+        process = pexpect.spawn(self.snapcraft_command, ['list-registered'])
+
+        for name, visibility, price, notes in expected_snaps:
+            # Ignores 'since' to avoid confusion on fake and actual stores.
+            process.expect(
+                '{} *[T:\-\d]+Z *{} *{} *{}'.format(
+                    name, visibility, price, notes))
+
+        process.expect(pexpect.EOF)
+        process.close()
+        return process.exitstatus
+
+    def update_name_arch_and_version(self, name=None, arch=None,
+                                     version=None):
         unique_id = uuid.uuid4().int
         if name is None:
             name = 'u1test-{}'.format(unique_id)
         if version is None:
             # The maximum size is 32 chars.
             version = str(unique_id)[:32]
-        updated_project_dir = self.copy_project_to_tmp(project_dir)
-        yaml_file = os.path.join(project_dir, 'snapcraft.yaml')
-        for line in fileinput.input(yaml_file, inplace=True):
+        if arch is None:
+            arch = 'amd64'
+        for line in fileinput.input('snapcraft.yaml', inplace=True):
+            if 'name: ' in line:
+                print('name: {}'.format(name))
+            elif 'version: ' in line:
+                print('version: {}'.format(version))
+            elif 'architectures: ' in line:
+                print('architectures: [{}]'.format(arch))
+            else:
+                print(line)
+
+    def update_name_and_version(self, name=None, version=None):
+        unique_id = uuid.uuid4().int
+        if name is None:
+            name = 'u1test-{}'.format(unique_id)
+        if version is None:
+            # The maximum size is 32 chars.
+            version = str(unique_id)[:32]
+        for line in fileinput.input('snapcraft.yaml', inplace=True):
             if 'name: ' in line:
                 print('name: {}'.format(name))
             elif 'version: ' in line:
                 print('version: {}'.format(version))
             else:
                 print(line)
-        return updated_project_dir
 
-    def gated(self, snap_name, expected_validations=[], expected_error=None):
+    def gated(self, snap_name, expected_validations=[], expected_output=None):
         process = pexpect.spawn(self.snapcraft_command, ['gated', snap_name])
 
-        if expected_error:
-            process.expect(expected_error)
+        if expected_output:
+            process.expect(expected_output)
         else:
             for name, revision in expected_validations:
                 process.expect('{} *{}'.format(name, revision))
@@ -230,3 +342,25 @@ class StoreTestCase(TestCase):
         process.expect(pexpect.EOF)
         process.close()
         return process.exitstatus
+
+    def close(self, *args, **kwargs):
+        process = pexpect.spawn(
+            self.snapcraft_command, ['close'] + list(args))
+        expected = kwargs.get('expected')
+        if expected is not None:
+            process.expect(expected)
+        process.expect(pexpect.EOF)
+        process.close()
+        return process.exitstatus
+
+    def push(self, snap, release=None, expected=None):
+        actions = ['push', snap]
+        if release is not None:
+            actions += ['--release', release]
+        process = pexpect.spawn(
+            self.snapcraft_command, actions)
+        if expected is not None:
+            process.expect(expected)
+        process.expect(pexpect.EOF)
+        process.close()
+        return process.exitstatus
