@@ -1,6 +1,6 @@
 # -*- Mode:Python; indent-tabs-mode:nil; tab-width:4 -*-
 #
-# Copyright (C) 2015 Canonical Ltd
+# Copyright (C) 2015-2017 Canonical Ltd
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License version 3 as
@@ -14,8 +14,6 @@
 # You should have received a copy of the GNU General Public License
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
-import os
-
 from testtools.matchers import (
     DirExists,
     Not
@@ -27,23 +25,28 @@ import integration_tests
 class CleanTestCase(integration_tests.TestCase):
 
     def test_clean(self):
-        project_dir = 'simple-make'
-        self.run_snapcraft('snap', project_dir)
+        self.copy_project_to_cwd('make-hello')
+        self.run_snapcraft('snap')
 
-        snap_dirs = ('stage', 'parts', 'prime')
+        snap_dirs = (self.parts_dir, self.stage_dir, self.prime_dir)
         for dir_ in snap_dirs:
-            self.assertThat(
-                os.path.join(project_dir, dir_), DirExists())
+            self.assertThat(dir_, DirExists())
 
-        self.run_snapcraft('clean', project_dir)
+        self.run_snapcraft('clean')
         for dir_ in snap_dirs:
-            self.assertThat(
-                os.path.join(project_dir, dir_), Not(DirExists()))
+            self.assertThat(dir_, Not(DirExists()))
 
     def test_clean_again(self):
         # Clean a second time doesn't fail.
         # Regression test for https://bugs.launchpad.net/snapcraft/+bug/1497371
-        project_dir = 'simple-make'
-        self.run_snapcraft('snap', project_dir)
-        self.run_snapcraft('clean', project_dir)
-        self.run_snapcraft('clean', project_dir)
+        self.copy_project_to_cwd('make-hello')
+        self.run_snapcraft('snap')
+        self.run_snapcraft('clean')
+        self.run_snapcraft('clean')
+
+    # Regression test for LP: #1596596
+    def test_clean_invalid_yaml(self):
+        self.run_snapcraft('clean', 'invalid-snap')
+        self.assertThat(self.parts_dir, Not(DirExists()))
+        self.assertThat(self.stage_dir, Not(DirExists()))
+        self.assertThat(self.prime_dir, Not(DirExists()))
