@@ -1,6 +1,6 @@
 # -*- Mode:Python; indent-tabs-mode:nil; tab-width:4 -*-
 #
-# Copyright (C) 2016 Canonical Ltd
+# Copyright (C) 2016-2017 Canonical Ltd
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License version 3 as
@@ -17,7 +17,9 @@
 import os
 
 from testtools.matchers import (
+    Contains,
     DirExists,
+    Equals,
     FileExists,
     Not
 )
@@ -30,12 +32,11 @@ class CleanBuildStepBuiltTestCase(integration_tests.TestCase):
     def setUp(self):
         super().setUp()
 
-        self.project_dir = 'independent-parts'
-        self.run_snapcraft('build', self.project_dir)
-        self.partsdir = os.path.join(self.project_dir, 'parts')
+        self.copy_project_to_cwd('independent-parts')
+        self.run_snapcraft('build')
         self.parts = {}
         for part in ['part1', 'part2']:
-            partdir = os.path.join(self.partsdir, part)
+            partdir = os.path.join(self.parts_dir, part)
             self.parts[part] = {
                 'partdir': partdir,
                 'sourcedir': os.path.join(partdir, 'src'),
@@ -54,22 +55,41 @@ class CleanBuildStepBuiltTestCase(integration_tests.TestCase):
     def test_clean_build_step(self):
         self.assert_files_exist()
 
-        self.run_snapcraft(['clean', '--step=build'], self.project_dir)
+        output = self.run_snapcraft(
+            ['clean', '--step=build'], debug=False)
 
         for part_name, part in self.parts.items():
             self.assertThat(part['builddir'], Not(DirExists()))
             self.assertThat(part['installdir'], Not(DirExists()))
             self.assertThat(part['sourcedir'], DirExists())
 
+        # Assert that the priming and staging areas were removed wholesale, not
+        # a part at a time (since we didn't specify any parts).
+        self.assertThat(output, Contains("Cleaning up priming area"))
+        self.assertThat(output, Contains("Cleaning up staging area"))
+
+        output = output.split('\n')
+        part1_output = [line.strip() for line in output if 'part1' in line]
+        part2_output = [line.strip() for line in output if 'part2' in line]
+        self.expectThat(part1_output, Equals([
+            'Skipping cleaning priming area for part1 (already clean)',
+            'Skipping cleaning staging area for part1 (already clean)',
+            'Cleaning build for part1'
+        ]))
+        self.expectThat(part2_output, Equals([
+            'Skipping cleaning priming area for part2 (already clean)',
+            'Skipping cleaning staging area for part2 (already clean)',
+            'Cleaning build for part2'
+        ]))
+
         # Now try to build again
-        self.run_snapcraft('build', self.project_dir)
+        self.run_snapcraft('build')
         self.assert_files_exist()
 
     def test_clean_build_step_single_part(self):
         self.assert_files_exist()
 
-        self.run_snapcraft(['clean', 'part1', '--step=build'],
-                           self.project_dir)
+        self.run_snapcraft(['clean', 'part1', '--step=build'])
         self.assertThat(self.parts['part1']['builddir'], Not(DirExists()))
         self.assertThat(self.parts['part1']['installdir'], Not(DirExists()))
         self.assertThat(self.parts['part1']['sourcedir'], DirExists())
@@ -82,7 +102,7 @@ class CleanBuildStepBuiltTestCase(integration_tests.TestCase):
             FileExists())
 
         # Now try to build again
-        self.run_snapcraft('build', self.project_dir)
+        self.run_snapcraft('build')
         self.assert_files_exist()
 
 
@@ -91,17 +111,14 @@ class CleanBuildStepPrimedTestCase(integration_tests.TestCase):
     def setUp(self):
         super().setUp()
 
-        self.project_dir = 'independent-parts'
-        self.run_snapcraft('prime', self.project_dir)
+        self.copy_project_to_cwd('independent-parts')
+        self.run_snapcraft('prime')
 
-        self.snapdir = os.path.join(self.project_dir, 'prime')
-        self.snap_bindir = os.path.join(self.snapdir, 'bin')
-        self.stagedir = os.path.join(self.project_dir, 'stage')
-        self.stage_bindir = os.path.join(self.stagedir, 'bin')
-        self.partsdir = os.path.join(self.project_dir, 'parts')
+        self.snap_bindir = os.path.join(self.prime_dir, 'bin')
+        self.stage_bindir = os.path.join(self.stage_dir, 'bin')
         self.parts = {}
         for part in ['part1', 'part2']:
-            partdir = os.path.join(self.partsdir, part)
+            partdir = os.path.join(self.parts_dir, part)
             self.parts[part] = {
                 'partdir': partdir,
                 'sourcedir': os.path.join(partdir, 'src'),
@@ -125,9 +142,9 @@ class CleanBuildStepPrimedTestCase(integration_tests.TestCase):
     def test_clean_build_step(self):
         self.assert_files_exist()
 
-        self.run_snapcraft(['clean', '--step=build'], self.project_dir)
-        self.assertThat(self.stagedir, Not(DirExists()))
-        self.assertThat(self.snapdir, Not(DirExists()))
+        self.run_snapcraft(['clean', '--step=build'])
+        self.assertThat(self.stage_dir, Not(DirExists()))
+        self.assertThat(self.prime_dir, Not(DirExists()))
 
         for part_name, part in self.parts.items():
             self.assertThat(part['builddir'], Not(DirExists()))
@@ -135,14 +152,13 @@ class CleanBuildStepPrimedTestCase(integration_tests.TestCase):
             self.assertThat(part['sourcedir'], DirExists())
 
         # Now try to prime again
-        self.run_snapcraft('prime', self.project_dir)
+        self.run_snapcraft('prime')
         self.assert_files_exist()
 
     def test_clean_build_step_single_part(self):
         self.assert_files_exist()
 
-        self.run_snapcraft(['clean', 'part1', '--step=build'],
-                           self.project_dir)
+        self.run_snapcraft(['clean', 'part1', '--step=build'])
         self.assertThat(os.path.join(self.stage_bindir, 'file1'),
                         Not(FileExists()))
         self.assertThat(os.path.join(self.stage_bindir, 'file2'), FileExists())
@@ -162,5 +178,5 @@ class CleanBuildStepPrimedTestCase(integration_tests.TestCase):
             FileExists())
 
         # Now try to prime again
-        self.run_snapcraft('prime', self.project_dir)
+        self.run_snapcraft('prime')
         self.assert_files_exist()
