@@ -1,6 +1,6 @@
 # -*- Mode:Python; indent-tabs-mode:nil; tab-width:4 -*-
 #
-# Copyright (C) 2016 Canonical Ltd
+# Copyright (C) 2016-2017 Canonical Ltd
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License version 3 as
@@ -16,7 +16,10 @@
 
 import os
 
-from testtools.matchers import FileExists
+from testtools.matchers import (
+    Equals,
+    FileExists
+)
 
 import integration_tests
 
@@ -24,12 +27,24 @@ import integration_tests
 class DebSourceTestCase(integration_tests.TestCase):
 
     def test_stage_deb(self):
-        project_dir = self.copy_project_to_tmp('simple-deb')
-        self.run_snapcraft('stage', project_dir)
+        self.copy_project_to_cwd('deb-hello')
+        self.run_snapcraft(['stage', 'deb'])
 
         self.assertThat(
-            os.path.join(project_dir, 'stage', 'bin', 'hello'),
+            os.path.join(self.stage_dir, 'bin', 'hello'),
             FileExists())
         self.assertThat(
-            os.path.join(project_dir, 'stage', 'usr', 'bin', 'world'),
+            os.path.join(self.stage_dir, 'usr', 'bin', 'world'),
             FileExists())
+
+    # Regression test for LP: #1634813
+    def test_stage_deb_with_symlink(self):
+        self.copy_project_to_cwd('deb-with-symlink')
+        self.run_snapcraft(['stage', 'deb-with-symlink'])
+
+        target = os.path.join(self.stage_dir, 'target')
+        symlink = os.path.join(self.stage_dir, 'symlink')
+        self.assertThat(target, FileExists())
+        self.assertThat(symlink, FileExists())
+        self.assertTrue(os.path.islink(symlink))
+        self.assertThat(os.readlink(symlink), Equals('target'))
