@@ -1,6 +1,6 @@
 # -*- Mode:Python; indent-tabs-mode:nil; tab-width:4 -*-
 #
-# Copyright (C) 2015, 2016 Canonical Ltd
+# Copyright (C) 2015-2017 Canonical Ltd
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License version 3 as
@@ -27,42 +27,33 @@ import integration_tests
 class MakePluginTestCase(integration_tests.TestCase):
 
     def test_stage_make_plugin(self):
-        project_dir = 'simple-make'
-        self.run_snapcraft('stage', project_dir)
+        self.run_snapcraft('stage', 'make-hello')
 
         binary_output = self.get_output_ignoring_non_zero_exit(
-            os.path.join('stage', 'bin', 'test'),
-            cwd=project_dir)
+            os.path.join(self.stage_dir, 'bin', 'test'))
         self.assertEqual('Hello world\n', binary_output)
 
     def test_clean_make_plugin(self):
-        project_dir = 'simple-make'
-        self.run_snapcraft('snap', project_dir)
+        self.run_snapcraft('snap', 'make-hello')
 
-        snap_dirs = ('stage', 'parts', 'prime')
+        snap_dirs = (self.parts_dir, self.stage_dir, self.prime_dir)
         for dir_ in snap_dirs:
-            self.assertThat(
-                os.path.join(project_dir, dir_), DirExists())
+            self.assertThat(dir_, DirExists())
 
-        self.run_snapcraft('clean', project_dir)
+        self.run_snapcraft('clean')
         for dir_ in snap_dirs:
-            self.assertThat(
-                os.path.join(project_dir, dir_), Not(DirExists()))
+            self.assertThat(dir_, Not(DirExists()))
 
     def test_nonstandard_makefile(self):
-        project_dir = 'simple-make-nonstandard-makefile'
-        self.run_snapcraft('stage', project_dir)
+        self.run_snapcraft('stage', 'make-with-nonstandard-makefile')
 
         binary_output = self.get_output_ignoring_non_zero_exit(
-            os.path.join('stage', 'bin', 'test'),
-            cwd=project_dir)
+            os.path.join(self.stage_dir, 'bin', 'test'))
         self.assertEqual('Hello world\n', binary_output)
 
     def test_stage_make_with_artifacts(self):
-        project_dir = 'simple-make-artifacts'
-        self.run_snapcraft('stage', project_dir)
+        self.run_snapcraft('stage', 'make-with-artifacts')
 
         binary_output = self.get_output_ignoring_non_zero_exit(
-            os.path.join('stage', 'test'),
-            cwd=project_dir)
+            os.path.join(self.stage_dir, 'test'))
         self.assertEqual('Hello World!\n', binary_output)
