@@ -1,6 +1,6 @@
 # -*- Mode:Python; indent-tabs-mode:nil; tab-width:4 -*-
 #
-# Copyright (C) 2015 Canonical Ltd
+# Copyright (C) 2015, 2017 Canonical Ltd
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License version 3 as
@@ -22,9 +22,8 @@ import integration_tests
 class SconsPluginTestCase(integration_tests.TestCase):
 
     def test_stage_scons_plugin(self):
-        project_dir = 'simple-scons'
-        self.run_snapcraft('stage', project_dir)
+        self.run_snapcraft('stage', 'scons-hello')
 
         binary_output = self.get_output_ignoring_non_zero_exit(
-            os.path.join('stage', 'opt', 'bin', 'main'), cwd=project_dir)
+            os.path.join(self.stage_dir, 'opt', 'bin', 'main'))
         self.assertEqual('Hello world\n', binary_output)
