@@ -1,7 +1,7 @@
 #!/bin/bash
 # -*- Mode:sh; indent-tabs-mode:nil; tab-width:4 -*-
 #
-# Copyright (C) 2015, 2016 Canonical Ltd
+# Copyright (C) 2015-2016 Canonical Ltd
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License version 3 as
@@ -48,16 +48,22 @@ parseargs(){
 python3 -m coverage && coverage="true"
 
 run_static_tests(){
-    SRC_PATHS="bin snapcraft integration_tests snaps_tests"
+    SRC_PATHS="bin snapcraft integration_tests snaps_tests external_snaps_tests"
     python3 -m flake8 --max-complexity=10 $SRC_PATHS
 }
 
 run_unit_tests(){
+    if [[ "$#" -lt 2 ]]; then
+        pattern="test_*.py"
+    else
+        pattern=$2
+    fi
+
     if [ ! -z "$coverage" ]; then
         python3 -m coverage erase
-        python3 -m coverage run --branch --source snapcraft -m unittest discover -b -v -s snapcraft -t .
+        python3 -m coverage run --branch --source snapcraft -m unittest discover -b -v -s snapcraft/tests -t . -p $pattern
     else
-        python3 -m unittest discover -b -v -s snapcraft -t .
+        python3 -m unittest discover -b -v -s snapcraft/tests -t . -p $pattern
     fi
 }
 
@@ -81,7 +87,7 @@ if [ ! -z "$RUN_STATIC" ] ; then
 fi
 
 if [ ! -z "$RUN_UNIT" ]; then
-    run_unit_tests
+    run_unit_tests "$@"
 fi
 
 if [ ! -z "$RUN_INTEGRATION" ]; then
