@@ -136,16 +136,97 @@ of the choice of plugin.
     parts is available to every snap author - just say 'after' and list the
     parts you want that others have already defined.
 
-  - build-packages: [deb, deb, deb...]
+  - build-packages: [pkg, pkg, pkg...]
 
-    A list of Ubuntu packages to install on the build host before building
-    the part. The files from these packages will not go into the final snap
-    unless they are also explicitly described in stage-packages.
+    A list of packages to install on the build host before building
+    the part. The files from these packages typically will not go into the
+    final snap unless they contain libraries that are direct dependencies of
+    binaries within the snap (in which case they'll be discovered via `ldd`),
+    or they are explicitly described in stage-packages.
 
-  - stage-packages: [deb, deb, deb...]
+  - stage-packages: YAML list
 
-    A list of Ubuntu packages must be unpacked in the `stage/` directory.
-    XXX before build? Before stage?
+    A set of packages to be downloaded and unpacked to join the part
+    before it's built. Note that these packages are not installed on the host.
+    Like the rest of the part, all files from these packages will make it into
+    the final snap unless filtered out via the `snap` keyword.
+
+    One may simply specify packages in a flat list, in which case the packages
+    will be fetched and unpacked regardless of build environment. In addition,
+    a specific grammar made up of sub-lists is supported here that allows one
+    to filter stage packages depending on various selectors (e.g. the target
+    arch), as well as specify optional packages. The grammar is made up of two
+    nestable statements: 'on' and 'try'.
+
+    Let's discuss `on`.
+
+        - on <selector>[,<selector>...]:
+          - ...
+        - else[ fail]:
+          - ...
+
+    The body of the 'on' clause is taken into account if every (AND, not OR)
+    selector is true for the target build environment. Currently the only
+    selectors supported are target architectures (e.g. amd64).
+
+    If the 'on' clause doesn't match and it's immediately followed by an 'else'
+    clause, the 'else' clause must be satisfied. An 'on' clause without an
+    'else' clause is considered satisfied even if no selector matched. The
+    'else fail' form allows erroring out if an 'on' clause was not matched.
+
+    For example, say you only wanted to stage `foo` if building for amd64 (and
+    not stage `foo` if otherwise):
+
+        - on amd64: [foo]
+
+    Building on that, say you wanted to stage `bar` if building on an arch
+    other than amd64:
+
+        - on amd64: [foo]
+        - else: [bar]
+
+    You can nest these for more complex behaviors:
+
+        - on amd64: [foo]
+        - else:
+          - on i386: [bar]
+          - on armhf: [baz]
+
+    If your project requires a package that is only available on amd64, you can
+    fail if you're not building for amd64:
+
+        - on amd64: [foo]
+        - else fail
+
+    Now let's discuss `try`:
+
+        - try:
+          - ...
+        - else:
+          - ...
+
+    The body of the 'try' clause is taken into account only when all packages
+    contained within it are valid. If not, if it's immediately followed by
+    'else' clauses they are tried in order, and one of them must be satisfied.
+    A 'try' clause with no 'else' clause is considered satisfied even if it
+    contains invalid packages.
+
+    For example, say you wanted to stage `foo`, but it wasn't available for all
+    architectures. Assuming your project builds without it, you can make it an
+    optional stage package:
+
+        - try: [foo]
+
+    You can also add alternatives:
+
+        - try: [foo]
+        - else: [bar]
+
+    Again, you can nest these for more complex behaviors:
+
+        - on amd64: [foo]
+        - else:
+          - try: [bar]
 
   - organize: YAML
 
@@ -221,18 +302,72 @@ of the choice of plugin.
     `prime/` directory reflects the file structure of the snap with no
     extraneous content).
 
+  - prepare: shell script
+
+    If present, the shell script defined here is run before the `build` step
+    of the plugin starts. The working directory is the base build
+    directory for the given part. The defined script is run with `/bin/sh`.
+
+    For example:
+
+      prepare: |
+        cd scripts
+        ./bootstrap.sh
+
+  - build: shell script
+
+    If present, the shell script defined here is run instead of the `build`
+    step of the plugin. The working directory is the base build directory
+    for the given part. The defined script is run with `/bin/sh`.
+
+    For example:
+
+      plugin: make
+      build: |
+        make project
+        make test
+        make special-install
+
+  - install: shell script
+
+    If present, the shell script defined here is run after the `build` step
+    of the plugin has finished. The working directory is the base build
+    directory for the given part. The defined script is run with `/bin/sh`.
+
+    For example:
+
+      install: |
+        sed -i 's|/usr/bin|$SNAP/usr/bin|g' my-bin-artifact.sh
+        mv my-bin-artifact.sh $SNAPCRAFT_PART_INSTALL/bin/my-bin-build.sh
+
+  - build-attributes: [attribute1, attribute2]
+
+    A list of special attributes that affect the build of this specific part.
+    Supported attributes:
+
+      - no-system-libraries:
+        Do not automatically copy required libraries from the system to satisfy
+        the dependencies of this part. This might be useful if one knows these
+        dependencies will be satisfied in other manner, e.g. via content
+        sharing from other snaps.
 """
 
 from collections import OrderedDict                 # noqa
+import pkg_resources                                # noqa
 import yaml                                         # noqa
 
 from snapcraft._baseplugin import BasePlugin        # noqa
 from snapcraft._options import ProjectOptions       # noqa
 from snapcraft._help import topic_help              # noqa
+# FIXME LP: #1662658
 from snapcraft._store import (                      # noqa
+    create_key,
+    close,
     download,
+    revisions,
     gated,
     list_keys,
+    list_registered,
     login,
     logout,
     push,
@@ -240,15 +375,27 @@ from snapcraft._store import (                      # noqa
     register_key,
     release,
     sign_build,
+    status,
     validate,
 )
 from snapcraft import common                        # noqa
 from snapcraft import plugins                       # noqa
 from snapcraft import sources                       # noqa
 from snapcraft import file_utils                    # noqa
+from snapcraft import shell_utils                   # noqa
 from snapcraft.internal import repo                 # noqa
 
 
+def _get_version():
+    try:
+        return pkg_resources.require('snapcraft')[0].version
+    except pkg_resources.DistributionNotFound:
+        return 'devel'
+
+
+__version__ = _get_version()
+
+
 # Setup yaml module globally
 # yaml OrderedDict loading and dumping
 # from http://stackoverflow.com/a/21048064 Wed Jun 22 16:05:34 UTC 2016
