@@ -15,11 +15,13 @@
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 import contextlib
-import glob
+import logging
 import os
-import shutil
 
-from snapcraft.internal import common, sources
+from snapcraft.internal import common
+
+
+logger = logging.getLogger(__name__)
 
 
 class BasePlugin:
@@ -38,55 +40,54 @@ class BasePlugin:
             '$schema': 'http://json-schema.org/draft-04/schema#',
             'type': 'object',
             'additionalProperties': False,
-            'properties': {
-                'source': {
-                    'type': 'string',
-                },
-                'source-type': {
-                    'type': 'string',
-                    'default': '',
-                },
-                'source-depth': {
-                    'type': 'integer',
-                    'default': 0,
-                },
-                'source-branch': {
-                    'type': 'string',
-                    'default': '',
-                },
-                'source-tag': {
-                    'type:': 'string',
-                    'default': '',
-                },
-                'source-subdir': {
-                    'type': 'string',
-                    'default': None,
-                },
-                'disable-parallel': {
-                    'type': 'boolean',
-                    'default': False,
-                },
-            },
-            'required': [
-                'source',
-            ],
-            'pull-properties': ['source', 'source-type', 'source-branch',
-                                'source-tag', 'source-subdir'],
-            'build-properties': ['disable-parallel']
+            'properties': {},
+            'pull-properties': [],
+            'build-properties': [],
+            'required': [],
         }
 
+    @classmethod
+    def get_pull_properties(cls):
+        schema_pull_properties = cls.schema().get('pull-properties', [])
+        if schema_pull_properties:
+            logger.warning(
+                'Use of pull-properties in the schema is deprecated.\n'
+                'Plugins should now implement get_pull_properties')
+            return schema_pull_properties
+
+        return []
+
+    @classmethod
+    def get_build_properties(cls):
+        schema_build_properties = cls.schema().get('build-properties', [])
+        if schema_build_properties:
+            logger.warning(
+                'Use of build-properties in the schema is deprecated.\n'
+                'Plugins should now implement get_build_properties')
+            return schema_build_properties
+
+        return []
+
     @property
     def PLUGIN_STAGE_SOURCES(self):
         """Define alternative sources.list."""
         return getattr(self, '_PLUGIN_STAGE_SOURCES', [])
 
+    @property
+    def stage_packages(self):
+        return self._stage_packages
+
+    @stage_packages.setter
+    def stage_packages(self, value):
+        self._stage_packages = value
+
     def __init__(self, name, options, project=None):
         self.name = name
         self.build_packages = []
-        self.stage_packages = []
+        self._stage_packages = []
 
         with contextlib.suppress(AttributeError):
-            self.stage_packages = options.stage_packages.copy()
+            self._stage_packages = options.stage_packages.copy()
         with contextlib.suppress(AttributeError):
             self.build_packages = options.build_packages.copy()
 
@@ -115,78 +116,20 @@ class BasePlugin:
 
     # The API
     def pull(self):
-        """Pull the source code and/or internal prereqs to build the part.
-
-        By default, the base implementation for pull will use the following
-        part properties to retrieve source code:
-
-        - source
-        - source-branch
-        - source-tag
-        - source-type
-
-        If source is empty or does not exist, the phase will be skipped.
-
-        Override or inherit from this method if you need to implement or
-        enhance with custom pull logic.
-        """
-        if getattr(self.options, 'source', None):
-            sources.get(self.sourcedir, self.build_basedir, self.options)
+        """Pull the source code and/or internal prereqs to build the part."""
+        pass
 
     def clean_pull(self):
-        """Clean the pulled source for this part.
-
-        The base implementation simply removes the sourcedir. Override this
-        method if your pull process was more involved and needs more cleaning.
-        """
-
-        if os.path.exists(self.sourcedir):
-            if os.path.islink(self.sourcedir):
-                os.remove(self.sourcedir)
-            else:
-                shutil.rmtree(self.sourcedir)
+        """Clean the pulled source for this part."""
+        pass
 
     def build(self):
-        """Build the source code retrieved from the pull phase.
-
-        The base implementation only copies sourcedir to build_basedir.
-        Override this method if you need to process the source code to make it
-        runnable.
-        """
-
-        if os.path.exists(self.build_basedir):
-            shutil.rmtree(self.build_basedir)
-
-        # FIXME: It's not necessary to ignore here anymore since it's now done
-        # in the Local source. However, it's left here so that it continues to
-        # work on old snapcraft trees that still have src symlinks.
-        def ignore(directory, files):
-            if directory is self.sourcedir:
-                snaps = glob.glob(os.path.join(directory, '*.snap'))
-                if snaps:
-                    snaps = [os.path.basename(s) for s in snaps]
-                    return common.SNAPCRAFT_FILES + snaps
-                else:
-                    return common.SNAPCRAFT_FILES
-            else:
-                return []
-
-        shutil.copytree(
-            self.sourcedir, self.build_basedir, symlinks=True, ignore=ignore)
+        """Build the source code retrieved from the pull phase."""
+        pass
 
     def clean_build(self):
-        """Clean the artifacts that resulted from building this part.
-
-        The base implementation simply removes the build_basedir and
-        installdir. Override this method if your build process was more
-        involved and needs more cleaning.
-        """
-
-        if os.path.exists(self.build_basedir):
-            shutil.rmtree(self.build_basedir)
-
-        if os.path.exists(self.installdir):
-            shutil.rmtree(self.installdir)
+        """Clean the artifacts that resulted from building this part."""
+        pass
 
     def snap_fileset(self):
         """Return a list of files to include or exclude in the resulting snap
