@@ -19,6 +19,10 @@ import multiprocessing
 import os
 import platform
 
+from snapcraft.internal import common
+from snapcraft.internal.deprecations import handle_deprecation_notice
+from snapcraft.internal.errors import SnapcraftEnvironmentError
+
 
 logger = logging.getLogger(__name__)
 
@@ -30,6 +34,7 @@ _ARCH_TRANSLATIONS = {
         'cross-compiler-prefix': 'arm-linux-gnueabihf-',
         'cross-build-packages': ['gcc-arm-linux-gnueabihf'],
         'triplet': 'arm-linux-gnueabihf',
+        'core-dynamic-linker': 'lib/ld-linux-armhf.so.3',
     },
     'aarch64': {
         'kernel': 'arm64',
@@ -37,6 +42,7 @@ _ARCH_TRANSLATIONS = {
         'cross-compiler-prefix': 'aarch64-linux-gnu-',
         'cross-build-packages': ['gcc-aarch64-linux-gnu'],
         'triplet': 'aarch64-linux-gnu',
+        'core-dynamic-linker': 'lib/ld-linux-aarch64.so.1',
     },
     'i686': {
         'kernel': 'x86',
@@ -49,11 +55,20 @@ _ARCH_TRANSLATIONS = {
         'cross-compiler-prefix': 'powerpc64le-linux-gnu-',
         'cross-build-packages': ['gcc-powerpc64le-linux-gnu'],
         'triplet': 'powerpc64le-linux-gnu',
+        'core-dynamic-linker': '/lib64/ld64.so.2',
+    },
+    'ppc': {
+        'kernel': 'powerpc',
+        'deb': 'powerpc',
+        'cross-compiler-prefix': 'powerpc-linux-gnu-',
+        'cross-build-packages': ['gcc-powerpc-linux-gnu'],
+        'triplet': 'powerpc-linux-gnu',
     },
     'x86_64': {
         'kernel': 'x86',
         'deb': 'amd64',
         'triplet': 'x86_64-linux-gnu',
+        'core-dynamic-linker': 'lib64/ld-linux-x86-64.so.2',
     },
     's390x': {
         'kernel': 's390x',
@@ -61,10 +76,29 @@ _ARCH_TRANSLATIONS = {
         'cross-compiler-prefix': 's390x-linux-gnu-',
         'cross-build-packages': ['gcc-s390x-linux-gnu'],
         'triplet': 's390x-linux-gnu',
+        'core-dynamic-linker': '/lib/ld64.so.1',
     }
 }
 
 
+_32BIT_USERSPACE_ARCHITECTURE = {
+    'aarch64': 'armv7l',
+    'armv8l': 'armv7l',
+    'ppc64le': 'ppc',
+    'x86_64': 'i686',
+}
+
+
+def _get_platform_architecture():
+    architecture = platform.machine()
+    if platform.architecture()[0] == '32bit':
+        userspace = _32BIT_USERSPACE_ARCHITECTURE.get(architecture)
+        if userspace:
+            architecture = userspace
+
+    return architecture
+
+
 class ProjectOptions:
 
     @property
@@ -89,7 +123,7 @@ class ProjectOptions:
 
     @property
     def is_cross_compiling(self):
-        return self.__target_machine != self.__host_machine
+        return self.__target_machine != self.__platform_arch
 
     @property
     def cross_compiler_prefix(self):
@@ -122,7 +156,11 @@ class ProjectOptions:
 
     @property
     def local_plugins_dir(self):
-        return os.path.join(self.parts_dir, 'plugins')
+        deprecated_plugins_dir = os.path.join(self.parts_dir, 'plugins')
+        if os.path.exists(deprecated_plugins_dir):
+            handle_deprecation_notice('dn2')
+            return deprecated_plugins_dir
+        return os.path.join(self.__project_dir, 'snap', 'plugins')
 
     @property
     def parts_dir(self):
@@ -150,10 +188,44 @@ class ProjectOptions:
         self._set_machine(target_deb_arch)
         self.__debug = debug
 
+    def get_core_dynamic_linker(self):
+        """Returns the dynamic linker used for the targetted core.
+        If not found realpath for `/lib/ld-linux.so.2` is returned.
+        However if core is not installed None will be returned.
+        """
+        core_path = common.get_core_path()
+        dynamic_linker_path = os.path.join(
+            core_path,
+            self.__machine_info.get('core-dynamic-linker',
+                                    'lib/ld-linux.so.2'))
+
+        # We can't use os.path.realpath because any absolute symlinks
+        # have to be interpreted relative to core_path, not the real
+        # root.
+        seen_paths = set()
+        while True:
+            if dynamic_linker_path in seen_paths:
+                raise SnapcraftEnvironmentError(
+                    "found symlink loop resolving dynamic linker path")
+
+            seen_paths.add(dynamic_linker_path)
+            if not os.path.lexists(dynamic_linker_path):
+                return None
+            if not os.path.islink(dynamic_linker_path):
+                return dynamic_linker_path
+
+            link_contents = os.readlink(dynamic_linker_path)
+            if os.path.isabs(link_contents):
+                dynamic_linker_path = os.path.join(
+                    core_path, link_contents.lstrip('/'))
+            else:
+                dynamic_linker_path = os.path.join(
+                    os.path.dirname(dynamic_linker_path), link_contents)
+
     def _set_machine(self, target_deb_arch):
-        self.__host_machine = platform.machine()
+        self.__platform_arch = _get_platform_architecture()
         if not target_deb_arch:
-            self.__target_machine = self.__host_machine
+            self.__target_machine = self.__platform_arch
         else:
             self.__target_machine = _find_machine(target_deb_arch)
             logger.info('Setting target machine to {!r}'.format(
