@@ -1,7 +1,7 @@
 #!/usr/bin/python3
 # -*- Mode:Python; indent-tabs-mode:nil; tab-width:4 -*-
 #
-# Copyright (C) 2016 Canonical Ltd
+# Copyright (C) 2016-2017 Canonical Ltd
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License version 3 as
@@ -20,17 +20,10 @@ import os
 import jsonschema
 import yaml
 
-from snapcraft.internal import common
-
-
-class SnapcraftSchemaError(Exception):
-
-    @property
-    def message(self):
-        return self._message
-
-    def __init__(self, message):
-        self._message = message
+from snapcraft.internal import (
+    common,
+    errors
+)
 
 
 class Validator:
@@ -54,6 +47,12 @@ class Validator:
         properties = sub['^(?!plugins$)[a-z0-9][a-z0-9+-\/]*$']['properties']
         return properties
 
+    @property
+    def definitions_schema(self):
+        """Return sub-schema that describes definitions used within schema."""
+
+        return self._schema['definitions'].copy()
+
     def _load_schema(self):
         schema_file = os.path.abspath(os.path.join(
             common.get_schemadir(), 'snapcraft.yaml'))
@@ -61,7 +60,7 @@ class Validator:
             with open(schema_file) as fp:
                 self._schema = yaml.load(fp)
         except FileNotFoundError:
-            raise SnapcraftSchemaError(
+            raise errors.SnapcraftSchemaError(
                 'snapcraft validation file is missing from installation path')
 
     def validate(self):
@@ -70,14 +69,4 @@ class Validator:
             jsonschema.validate(
                 self._snapcraft, self._schema, format_checker=format_check)
         except jsonschema.ValidationError as e:
-            messages = [e.message]
-            path = []
-            while e.absolute_path:
-                path.append(e.absolute_path.popleft())
-            if path:
-                messages.insert(0, "The '{}' property does not match the "
-                                   "required schema:".format('/'.join(path)))
-            if e.cause:
-                messages.append('({})'.format(e.cause))
-
-            raise SnapcraftSchemaError(' '.join(messages))
+            raise errors.SnapcraftSchemaError.from_validation_error(e)
