@@ -1,6 +1,6 @@
 # -*- Mode:Python; indent-tabs-mode:nil; tab-width:4 -*-
 #
-# Copyright (C) 2015, 2016 Canonical Ltd
+# Copyright (C) 2015-2016 Canonical Ltd
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License version 3 as
@@ -79,6 +79,11 @@ def run_output(cmd, **kwargs):
             return output.decode('latin-1', 'surrogateescape').strip()
 
 
+def get_core_path():
+    """Returns the path to the core snap."""
+    return os.path.join(os.path.sep, 'snap', 'core', 'current')
+
+
 def format_snap_name(snap):
     if 'arch' not in snap:
         snap['arch'] = snap.get('architectures', None)
@@ -107,6 +112,9 @@ def set_schemadir(schemadir):
 
 
 def get_schemadir():
+    snap = os.environ.get('SNAP')
+    if snap:
+        return os.path.join(snap, 'share', 'snapcraft', 'schema')
     return _schemadir
 
 
@@ -140,6 +148,9 @@ def set_tourdir(tourdir):
 
 
 def get_tourdir():
+    snap = os.environ.get('SNAP')
+    if snap:
+        return os.path.join(snap, 'tour')
     return _tourdir
 
 
@@ -154,8 +165,12 @@ def get_python2_path(root):
             'PYTHONPATH cannot be set for {!r}'.format(root))
 
 
+def get_url_scheme(url):
+    return urllib.parse.urlparse(url).scheme
+
+
 def isurl(url):
-    return urllib.parse.urlparse(url).scheme != ''
+    return get_url_scheme(url) != ''
 
 
 def reset_env():
@@ -229,7 +244,12 @@ def get_include_paths(root, arch_triplet):
     return [p for p in paths if os.path.exists(p)]
 
 
-def get_library_paths(root, arch_triplet):
+def get_library_paths(root, arch_triplet, existing_only=True):
+    """Returns common library paths for a snap.
+
+    If existing_only is set the paths returned must exist for
+    the root that was set.
+    """
     paths = [
         os.path.join(root, 'lib'),
         os.path.join(root, 'usr', 'lib'),
@@ -237,7 +257,10 @@ def get_library_paths(root, arch_triplet):
         os.path.join(root, 'usr', 'lib', arch_triplet),
     ]
 
-    return [p for p in paths if os.path.exists(p)]
+    if existing_only:
+        paths = [p for p in paths if os.path.exists(p)]
+
+    return paths
 
 
 def get_pkg_config_paths(root, arch_triplet):
@@ -253,18 +276,3 @@ def get_pkg_config_paths(root, arch_triplet):
     ]
 
     return [p for p in paths if os.path.exists(p)]
-
-
-def combine_paths(paths, prepend, separator):
-    paths = ['{}{}'.format(prepend, p) for p in paths]
-    return separator.join(paths)
-
-
-def format_path_variable(envvar, paths, prepend, separator):
-    if not paths:
-        raise ValueError(
-            "Failed to format '${}': no paths supplied".format(envvar))
-
-    return '{envvar}="${envvar}{separator}{paths}"'.format(
-        envvar=envvar, separator=separator, paths=combine_paths(
-            paths, prepend, separator))
