@@ -1,6 +1,6 @@
 # -*- Mode:Python; indent-tabs-mode:nil; tab-width:4 -*-
 #
-# Copyright (C) 2015, 2016 Canonical Ltd
+# Copyright (C) 2015-2017 Canonical Ltd
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License version 3 as
@@ -18,15 +18,16 @@ import contextlib
 import logging
 import os
 import shutil
-import sys
 import tarfile
 import time
-from subprocess import Popen, PIPE, STDOUT
+from subprocess import check_call, Popen, PIPE, STDOUT
+from tempfile import TemporaryDirectory
 
 import yaml
 from progressbar import AnimatedMarker, ProgressBar
 
 import snapcraft
+from snapcraft import formatting_utils
 import snapcraft.internal
 from snapcraft.internal import (
     common,
@@ -35,6 +36,8 @@ from snapcraft.internal import (
     pluginhandler,
     repo,
 )
+from snapcraft.internal.cache import SnapCache
+from snapcraft.internal.indicators import is_dumb_terminal
 from snapcraft.internal.project_loader import replace_attr
 
 
@@ -64,18 +67,26 @@ _STEPS_TO_AUTOMATICALLY_CLEAN_IF_DIRTY = {'stage', 'prime'}
 
 def init():
     """Initialize a snapcraft project."""
+    snapcraft_yaml_path = os.path.join('snap', 'snapcraft.yaml')
 
-    if os.path.exists('snapcraft.yaml'):
+    if os.path.exists(snapcraft_yaml_path):
+        raise EnvironmentError(
+            '{} already exists!'.format(snapcraft_yaml_path))
+    elif os.path.exists('snapcraft.yaml'):
         raise EnvironmentError('snapcraft.yaml already exists!')
     elif os.path.exists('.snapcraft.yaml'):
         raise EnvironmentError('.snapcraft.yaml already exists!')
     yaml = _TEMPLATE_YAML.strip()
-    with open('snapcraft.yaml', mode='w+') as f:
+    with contextlib.suppress(FileExistsError):
+        os.mkdir(os.path.dirname(snapcraft_yaml_path))
+    with open(snapcraft_yaml_path, mode='w') as f:
         f.write(yaml)
-    logger.info('Created snapcraft.yaml.')
+    logger.info('Created {}.'.format(snapcraft_yaml_path))
     logger.info(
         'Edit the file to your liking or run `snapcraft` to get started')
 
+    return snapcraft_yaml_path
+
 
 def execute(step, project_options, part_names=None):
     """Execute until step in the lifecycle for part_names or all parts.
@@ -97,7 +108,11 @@ def execute(step, project_options, part_names=None):
     :returns: A dict with the snap name, version, type and architectures.
     """
     config = snapcraft.internal.load_config(project_options)
-    repo.install_build_packages(config.build_tools)
+    repo.Repo.install_build_packages(config.build_tools)
+
+    if (os.environ.get('SNAPCRAFT_SETUP_CORE') and
+            config.data['confinement'] == 'classic'):
+        _setup_core(project_options.deb_arch)
 
     _Executor(config, project_options).run(step, part_names)
 
@@ -107,6 +122,40 @@ def execute(step, project_options, part_names=None):
             'type': config.data.get('type', '')}
 
 
+def _setup_core(deb_arch):
+    core_path = common.get_core_path()
+    if os.path.exists(core_path) and os.listdir(core_path):
+        logger.debug('{!r} already exists, skipping core setup'.format(
+            core_path))
+        return
+    snap_cache = SnapCache(project_name='snapcraft-core')
+
+    # Try to get the latest revision.
+    core_snap = snap_cache.get(deb_arch=deb_arch)
+    if core_snap:
+        # The current hash matches the filename
+        current_hash = os.path.splitext(os.path.basename(core_snap))[0]
+    else:
+        current_hash = ''
+
+    with TemporaryDirectory() as d:
+        download_path = os.path.join(d, 'core.snap')
+        download_hash = snapcraft.download('core', 'stable', download_path,
+                                           deb_arch, except_hash=current_hash)
+        if download_hash != current_hash:
+            snap_cache.cache(snap_filename=download_path)
+            snap_cache.prune(deb_arch=deb_arch, keep_hash=download_hash)
+
+    core_snap = snap_cache.get(deb_arch=deb_arch)
+
+    # Now unpack
+    logger.info('Setting up {!r} in {!r}'.format(core_snap, core_path))
+    if os.path.exists(core_path) and not os.listdir(core_path):
+        check_call(['sudo', 'rmdir', core_path])
+    check_call(['sudo', 'mkdir', '-p', os.path.dirname(core_path)])
+    check_call(['sudo', 'unsquashfs', '-d', core_path, core_snap])
+
+
 def _replace_in_part(part):
     for key, value in part.code.options.__dict__.items():
         value = replace_attr(value, [
@@ -131,8 +180,9 @@ class _Executor:
         for part in self.config.all_parts:
             steps_run[part.name] = set()
             for step in common.COMMAND_ORDER:
-                if part.is_dirty(step):
-                    self._handle_dirty(part, step)
+                dirty_report = part.get_dirty_report(step)
+                if dirty_report:
+                    self._handle_dirty(part, step, dirty_report)
                 elif not (part.should_step_run(step)):
                     steps_run[part.name].add(step)
                     part.notify_part_progress('Skipping {}'.format(step),
@@ -199,15 +249,39 @@ class _Executor:
         if step == 'prime' and part_names == self.config.part_names:
             common.env = self.config.snap_env()
             meta.create_snap_packaging(self.config.data,
-                                       self.project_options.snap_dir,
-                                       self.project_options.parts_dir)
+                                       self.project_options)
 
-    def _handle_dirty(self, part, step):
+    def _handle_dirty(self, part, step, dirty_report):
         if step not in _STEPS_TO_AUTOMATICALLY_CLEAN_IF_DIRTY:
-            raise RuntimeError(
-                'The {0!r} step of {1!r} is out of date. Please clean that '
-                "part's {0!r} step in order to rebuild".format(
+            message_components = [
+                'The {!r} step of {!r} is out of date:\n\n'.format(
+                    step, part.name)]
+
+            if dirty_report.dirty_properties:
+                humanized_properties = formatting_utils.humanize_list(
+                    dirty_report.dirty_properties, 'and')
+                pluralized_connection = formatting_utils.pluralize(
+                    dirty_report.dirty_properties, 'property appears',
+                    'properties appear')
+                message_components.append(
+                    'The {} part {} to have changed.\n'.format(
+                        humanized_properties, pluralized_connection))
+
+            if dirty_report.dirty_project_options:
+                humanized_options = formatting_utils.humanize_list(
+                    dirty_report.dirty_project_options, 'and')
+                pluralized_connection = formatting_utils.pluralize(
+                    dirty_report.dirty_project_options, 'option appears',
+                    'options appear')
+                message_components.append(
+                    'The {} project {} to have changed.\n'.format(
+                        humanized_options, pluralized_connection))
+
+            message_components.append(
+                "\nIn order to continue, please clean that part's {0!r} step "
+                "by running: snapcraft clean {1} -s {0}\n".format(
                     step, part.name))
+            raise RuntimeError(''.join(message_components))
 
         staged_state = self.config.get_project_state('stage')
         primed_state = self.config.get_project_state('prime')
@@ -222,14 +296,17 @@ class _Executor:
             for dependent in self.config.all_parts:
                 if (dependent.name in dependents and
                         not dependent.is_clean('build')):
-                    humanized_parts = _humanize_list(dependents)
+                    humanized_parts = formatting_utils.humanize_list(
+                        dependents, 'and')
+                    pluralized_depends = formatting_utils.pluralize(
+                        dependents, "depends", "depend")
 
                     raise RuntimeError(
                         'The {0!r} step for {1!r} needs to be run again, but '
-                        '{2} depend{3} upon it. Please clean the build '
+                        '{2} {3} upon it. Please clean the build '
                         'step of {2} first.'.format(
                             step, part.name, humanized_parts,
-                            's' if len(dependents) == 1 else ''))
+                            pluralized_depends))
 
         part.clean(staged_state, primed_state, step, '(out of date)')
 
@@ -239,7 +316,7 @@ def _create_tar_filter(tar_filename):
         fn = tarinfo.name
         if fn.startswith('./parts/') and not fn.startswith('./parts/plugins'):
             return None
-        elif fn in ('./stage', './prime', './snap', tar_filename):
+        elif fn in ('./stage', './prime', tar_filename):
             return None
         elif fn.endswith('.snap'):
             return None
@@ -247,15 +324,7 @@ def _create_tar_filter(tar_filename):
     return _tar_filter
 
 
-def cleanbuild(project_options):
-    if not repo.is_package_installed('lxd'):
-        raise EnvironmentError(
-            'The lxd package is not installed, in order to use `cleanbuild` '
-            'you must install lxd onto your system. Refer to the '
-            '"Ubuntu Desktop and Ubuntu Server" section on '
-            'https://linuxcontainers.org/lxd/getting-started-cli/'
-            '#ubuntu-desktop-and-ubuntu-server to enable a proper setup.')
-
+def cleanbuild(project_options, remote=''):
     config = snapcraft.internal.load_config(project_options)
     tar_filename = '{}_{}_source.tar.bz2'.format(
         config.data['name'], config.data['version'])
@@ -264,7 +333,8 @@ def cleanbuild(project_options):
         t.add(os.path.curdir, filter=_create_tar_filter(tar_filename))
 
     snap_filename = common.format_snap_name(config.data)
-    lxd.Cleanbuilder(snap_filename, tar_filename, project_options).execute()
+    lxd.Cleanbuilder(snap_filename, tar_filename, project_options,
+                     remote=remote).execute()
 
 
 def _snap_data_from_dir(directory):
@@ -306,7 +376,10 @@ def snap(project_options, directory=None, output=None):
     with Popen(['mksquashfs', snap_dir, snap_name] + mksquashfs_args,
                stdout=PIPE, stderr=STDOUT) as proc:
         ret = None
-        if os.isatty(sys.stdout.fileno()):
+        if is_dumb_terminal():
+            logger.info('Snapping {!r} ...'.format(snap['name']))
+            ret = proc.wait()
+        else:
             message = '\033[0;32m\rSnapping {!r}\033[0;32m '.format(
                 snap['name'])
             progress_indicator = ProgressBar(
@@ -324,9 +397,6 @@ def snap(project_options, directory=None, output=None):
                 count += 1
                 time.sleep(.2)
                 ret = proc.poll()
-        else:
-            logger.info('Snapping {!r} ...'.format(snap['name']))
-            ret = proc.wait()
         print('')
         if ret != 0:
             logger.error(proc.stdout.read().decode('utf-8'))
@@ -335,6 +405,7 @@ def snap(project_options, directory=None, output=None):
         logger.debug(proc.stdout.read().decode('utf-8'))
 
     logger.info('Snapped {}'.format(snap_name))
+    return snap_name
 
 
 def _reverse_dependency_tree(config, part_name):
@@ -361,22 +432,6 @@ def _clean_part_and_all_dependents(part_name, step, config, staged_state,
     config.parts.clean_part(part_name, staged_state, primed_state, step)
 
 
-def _humanize_list(items):
-    if len(items) == 0:
-        return ''
-
-    quoted_items = ['{!r}'.format(item) for item in sorted(items)]
-    if len(items) == 1:
-        return quoted_items[0]
-
-    humanized = ', '.join(quoted_items[:-1])
-
-    if len(items) > 2:
-        humanized += ','
-
-    return '{} and {}'.format(humanized, quoted_items[-1])
-
-
 def _verify_dependents_will_be_cleaned(part_name, clean_part_names, step,
                                        config):
     # Get the name of the parts that depend upon this one
@@ -386,7 +441,8 @@ def _verify_dependents_will_be_cleaned(part_name, clean_part_names, step,
     if not dependents.issubset(clean_part_names):
         for part in config.all_parts:
             if part.name in dependents and not part.is_clean(step):
-                humanized_parts = _humanize_list(dependents)
+                humanized_parts = formatting_utils.humanize_list(
+                    dependents, 'and')
 
                 raise RuntimeError(
                     'Requested clean of {!r} but {} depend{} upon it. Please '
@@ -416,10 +472,6 @@ def _remove_directory_if_empty(directory):
 
 
 def _cleanup_common_directories(config, project_options):
-    _remove_directory_if_empty(project_options.parts_dir)
-    _remove_directory_if_empty(project_options.stage_dir)
-    _remove_directory_if_empty(project_options.snap_dir)
-
     max_index = -1
     for part in config.all_parts:
         step = part.last_step()
@@ -428,32 +480,83 @@ def _cleanup_common_directories(config, project_options):
             if index > max_index:
                 max_index = index
 
-    # If no parts have been pulled, remove the parts directory. In most cases
-    # this directory should have already been cleaned, but this handles the
-    # case of a failed pull. Note however that the presence of local plugins
-    # should prevent this removal.
-    if (max_index < common.COMMAND_ORDER.index('pull') and
-            os.path.exists(project_options.parts_dir) and not
-            os.path.exists(project_options.local_plugins_dir)):
+    with contextlib.suppress(IndexError):
+        _cleanup_common_directories_for_step(
+            common.COMMAND_ORDER[max_index+1], project_options)
+
+
+def _cleanup_common_directories_for_step(step, project_options, parts=None):
+    if not parts:
+        parts = []
+
+    index = common.COMMAND_ORDER.index(step)
+
+    if index <= common.COMMAND_ORDER.index('prime'):
+        # Remove the priming area.
+        _cleanup_common(
+            project_options.snap_dir, 'prime', 'Cleaning up priming area',
+            parts)
+
+    if index <= common.COMMAND_ORDER.index('stage'):
+        # Remove the staging area.
+        _cleanup_common(
+            project_options.stage_dir, 'stage', 'Cleaning up staging area',
+            parts)
+
+    if index <= common.COMMAND_ORDER.index('pull'):
+        # Remove the parts directory (but leave local plugins alone).
+        _cleanup_parts_dir(
+            project_options.parts_dir, project_options.local_plugins_dir,
+            parts)
+
+    _remove_directory_if_empty(project_options.snap_dir)
+    _remove_directory_if_empty(project_options.stage_dir)
+    _remove_directory_if_empty(project_options.parts_dir)
+
+
+def _cleanup_common(directory, step, message, parts):
+    if os.path.isdir(directory):
+        logger.info(message)
+        shutil.rmtree(directory)
+    for part in parts:
+        part.mark_cleaned(step)
+
+
+def _cleanup_parts_dir(parts_dir, local_plugins_dir, parts):
+    if os.path.exists(parts_dir):
         logger.info('Cleaning up parts directory')
-        shutil.rmtree(project_options.parts_dir)
-
-    # If no parts have been staged, remove staging area.
-    should_remove_stagedir = max_index < common.COMMAND_ORDER.index('stage')
-    if should_remove_stagedir and os.path.exists(project_options.stage_dir):
-        logger.info('Cleaning up staging area')
-        shutil.rmtree(project_options.stage_dir)
-
-    # If no parts have been primed, remove snapping area.
-    should_remove_snapdir = max_index < common.COMMAND_ORDER.index('prime')
-    if should_remove_snapdir and os.path.exists(project_options.snap_dir):
-        logger.info('Cleaning up snapping area')
-        shutil.rmtree(project_options.snap_dir)
+        for subdirectory in os.listdir(parts_dir):
+            path = os.path.join(parts_dir, subdirectory)
+            if path != local_plugins_dir:
+                try:
+                    shutil.rmtree(path)
+                except NotADirectoryError:
+                    os.remove(path)
+    for part in parts:
+        part.mark_cleaned('build')
+        part.mark_cleaned('pull')
 
 
 def clean(project_options, parts, step=None):
+    # step defaults to None because that's how it comes from docopt when it's
+    # not set.
+    if not step:
+        step = 'pull'
+
+    if not parts and step == 'pull':
+        _cleanup_common_directories_for_step(step, project_options)
+        return
+
     config = snapcraft.internal.load_config()
 
+    if not parts and (step == 'stage' or step == 'prime'):
+        # If we've been asked to clean stage or prime without being given
+        # specific parts, just blow away those directories instead of
+        # doing it per part.
+        _cleanup_common_directories_for_step(
+            step, project_options, parts=config.all_parts)
+        return
+
     if parts:
         config.parts.validate(parts)
     else:
