@@ -1,7 +1,7 @@
 #!/usr/bin/python3
 # -*- Mode:Python; indent-tabs-mode:nil; tab-width:4 -*-
 #
-# Copyright (C) 2016 Canonical Ltd
+# Copyright (C) 2016-2017 Canonical Ltd
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License version 3 as
@@ -17,12 +17,14 @@
 
 import logging
 import os
+import sys
 from contextlib import contextmanager
-from subprocess import check_call, CalledProcessError
+from subprocess import check_call, check_output, CalledProcessError
 from time import sleep
 
 import petname
 
+from snapcraft.internal.errors import SnapcraftEnvironmentError
 
 logger = logging.getLogger(__name__)
 
@@ -34,12 +36,17 @@ _PROXY_KEYS = ['http_proxy', 'https_proxy', 'no_proxy', 'ftp_proxy']
 
 class Cleanbuilder:
 
-    def __init__(self, snap_output, tar_filename, project_options):
+    def __init__(self, snap_output, tar_filename, project_options,
+                 remote=None):
         self._snap_output = snap_output
         self._tar_filename = tar_filename
         self._project_options = project_options
-        self._container_name = 'snapcraft-{}'.format(
-            petname.Generate(3, '-'))
+        container_name = 'snapcraft-{}'.format(petname.Generate(3, '-'))
+
+        if not remote:
+            remote = _get_default_remote()
+        _verify_remote(remote)
+        self._container_name = '{}:{}'.format(remote, container_name)
 
     def _push_file(self, src, dst):
         check_call(['lxc', 'file', 'push',
@@ -59,6 +66,9 @@ class Cleanbuilder:
                 'lxc', 'launch', '-e',
                 'ubuntu:xenial/{}'.format(self._project_options.deb_arch),
                 self._container_name])
+            check_call([
+                'lxc', 'config', 'set', self._container_name,
+                'environment.SNAPCRAFT_SETUP_CORE', '1'])
             yield
         finally:
             # Stopping takes a while and lxc doesn't print anything.
@@ -108,3 +118,47 @@ class Cleanbuilder:
                 if retry_count == 0:
                     raise e
         logger.info('Network connection established')
+
+
+def _get_default_remote():
+    """Query and return the default lxd remote.
+
+    Use the lxc command to query for the default lxd remote. In most
+    cases this will return the local remote.
+
+    :returns: default lxd remote.
+    :rtype: string.
+    :raises snapcraft.internal.errors.SnapcraftEnvironmentError:
+        raised if the lxc call fails.
+    """
+    try:
+        default_remote = check_output(['lxc', 'remote', 'get-default'])
+    except CalledProcessError:
+        raise SnapcraftEnvironmentError(
+            'You must have LXD installed in order to use cleanbuild. '
+            'However, it is either not installed or not configured '
+            'properly.\n'
+            'Refer to the documentation at '
+            'https://linuxcontainers.org/lxd/getting-started-cli.')
+    return default_remote.decode(sys.getfilesystemencoding()).strip()
+
+
+def _verify_remote(remote):
+    """Verify that the lxd remote exists.
+
+    :param str remote: the lxd remote to verify.
+    :raises snapcraft.internal.errors.SnapcraftEnvironmentError:
+        raised if the lxc call listing the remote fails.
+    """
+    # There is no easy way to grep the results from `lxc remote list`
+    # so we try and execute a simple operation against the remote.
+    try:
+        check_output(['lxc', 'list', '{}:'.format(remote)])
+    except CalledProcessError as e:
+        raise SnapcraftEnvironmentError(
+            'There are either no permissions or the remote {!r} '
+            'does not exist.\n'
+            'Verify the existing remotes by running `lxc remote list`\n'
+            'To setup a new remote, follow the instructions at\n'
+            'https://linuxcontainers.org/lxd/getting-started-cli/'
+            '#multiple-hosts'.format(remote)) from e
