@@ -1,7 +1,7 @@
 #!/usr/bin/python3
 # -*- Mode:Python; indent-tabs-mode:nil; tab-width:4 -*-
 #
-# Copyright (C) 2015, 2016 Canonical Ltd
+# Copyright (C) 2015-2016 Canonical Ltd
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License version 3 as
@@ -28,24 +28,34 @@ Usage:
   snapcraft [options] strip [<part> ...]
   snapcraft [options] clean [<part> ...] [--step <step>]
   snapcraft [options] snap [<directory> --output <snap-file>]
-  snapcraft [options] cleanbuild
+  snapcraft [options] cleanbuild [--remote=<remote>]
   snapcraft [options] login
   snapcraft [options] logout
+  snapcraft [options] list-registered
+  snapcraft [options] registered
   snapcraft [options] list-keys
   snapcraft [options] keys
+  snapcraft [options] create-key [<key-name>]
   snapcraft [options] register-key [<key-name>]
   snapcraft [options] register <snap-name> [--private]
   snapcraft [options] sign-build <snap-file> [--key-name=<key-name>] [--local]
   snapcraft [options] upload <snap-file>
   snapcraft [options] push <snap-file> [--release <channels>]
   snapcraft [options] release <snap-name> <revision> <channel>
+  snapcraft [options] status <snap-name> [--series=<series>] [--arch=<arch>]
+  snapcraft [options] list-revisions <snap-name> [--series=<series>] [--arch=<arch>]
+  snapcraft [options] revisions <snap-name> [--series=<series>] [--arch=<arch>]
+  snapcraft [options] history <snap-name> [--series=<series>] [--arch=<arch>]
+  snapcraft [options] close <snap-name> <channel_names>...
   snapcraft [options] list-plugins
+  snapcraft [options] plugins
   snapcraft [options] tour [<directory>]
   snapcraft [options] update
   snapcraft [options] gated <snap-name>
   snapcraft [options] validate <snap-name> <validation>... [--key-name=<key-name>]
   snapcraft [options] define <part-name>
   snapcraft [options] search [<query> ...]
+  snapcraft [options] enable-ci [<ci-system>] [--refresh]
   snapcraft [options] help (topics | <plugin> | <topic>) [--devel]
   snapcraft (-h | --help)
   snapcraft --version
@@ -61,6 +71,11 @@ Options:
                                         architecture. Very few plugins support
                                         this.
 
+Options specific to cleanbuild:
+  --remote <remote> Use a specific lxd remote to run the cleanbuild on.
+                    This requires prior setup which is described on:
+                    https://linuxcontainers.org/lxd/getting-started-cli/#multiple-hosts
+
 Options specific to pulling:
   --enable-geoip         enables geoip for the pull step if stage-packages
                          are used.
@@ -81,24 +96,36 @@ Options specific to snapping:
 
 Options specific to store interaction:
   --release <channels>  Comma separated list of channels to release to.
+  --series <series>     Snap series [default: {DEFAULT_SERIES}].
 
 The available commands are:
-  help         Obtain help for a certain plugin or topic
-  init         Initialize a snapcraft project.
-  list-plugins List the available plugins that handle different types of part.
-  login        Authenticate session against Ubuntu One SSO.
-  logout       Clear session credentials.
-  list-keys    List keys available for signing snaps.
-  keys         Alias for list-keys.
-  register-key Register a key for signing snaps.
-  register     Register the package name in the store.
-  tour         Setup the snapcraft examples tour in the specified directory,
-               or ./snapcraft-tour/.
-  sign-build   Sign a built snap file and assert it using the developer's key.
-  push         Pushes and optionally releases a snap to the Ubuntu Store.
-  upload       DEPRECATED Upload a snap to the Ubuntu Store. The push command
-               supersedes this command.
-  release      Release a revision of a snap to a specific channel.
+  help            Obtain help for a certain plugin or topic
+  init            Initialize a snapcraft project.
+  list-plugins    List the available plugins that handle different types of part.
+  plugins         Alias for list-plugins.
+  login           Authenticate session against Ubuntu One SSO.
+  logout          Clear session credentials.
+  list-registered List snap names registered or shared with you.
+  registered      Alias for list-registered.
+  list-keys       List keys available for signing snaps.
+  keys            Alias for list-keys.
+  create-key      Create a key pair for signing snaps.
+  register-key    Register a key for signing snaps.
+  register        Register the package name in the store.
+  tour            Setup the snapcraft examples tour in the specified directory,
+                  or ./snapcraft-tour/.
+  sign-build      Sign a built snap file and assert it using the developer's key.
+  push            Pushes and optionally releases a snap to the Ubuntu Store.
+  upload          DEPRECATED Upload a snap to the Ubuntu Store. The push command
+                  supersedes this command.
+  release         Release a revision of a snap to a specific channel.
+  status          Show the current status of a snap per channel and architecture.
+  list-revisions  List all revisions of a snap.
+  revisions       Alias for list-revisions.
+  history         Deprecated alias for list-revisions.
+  close           Close one or more channels of a snap.
+  enable-ci       EXPERIMENTAL enable continuous-integration systems to build and
+                  release snaps to the Ubuntu Store.
 
 The available lifecycle commands are:
   clean        Remove content - cleans downloads, builds or install artifacts.
@@ -125,12 +152,11 @@ https://linuxcontainers.org/lxd/getting-started-cli
 to get started.
 
 For more help, visit the documentation:
-http://developer.ubuntu.com/snappy/snapcraft
+http://snapcraft.io/docs/build-snaps
 """  # NOQA
 
 import logging
 import os
-import pkg_resources
 import pkgutil
 import shutil
 import sys
@@ -138,24 +164,24 @@ import sys
 from docopt import docopt
 
 import snapcraft
-from snapcraft.internal import lifecycle, log, parts
+from snapcraft.integrations import enable_ci
+from snapcraft.internal import (
+    deprecations,
+    lifecycle,
+    log,
+    parts,
+)
 from snapcraft.internal.common import (
     format_output_in_columns,
     get_terminal_width,
     get_tourdir)
+from snapcraft.storeapi.constants import DEFAULT_SERIES
 
 
 logger = logging.getLogger(__name__)
 _SNAPCRAFT_TOUR_DIR = "./snapcraft-tour/"
 
 
-def _get_version():
-    try:
-        return pkg_resources.require('snapcraft')[0].version
-    except pkg_resources.DistributionNotFound:
-        return 'devel'
-
-
 def _scaffold_examples(directory):
     logger.debug("Copying examples tour to {}".format(directory))
     dest_dir = os.path.abspath(directory)
@@ -206,7 +232,8 @@ def _get_project_options(args):
 
 
 def main(argv=None):
-    args = docopt(__doc__, version=_get_version(), argv=argv)
+    doc = __doc__.format(DEFAULT_SERIES=DEFAULT_SERIES)
+    args = docopt(doc, version=snapcraft.__version__, argv=argv)
 
     # Default log level is INFO unless --debug is specified
     log_level = logging.INFO
@@ -216,6 +243,9 @@ def main(argv=None):
     log.configure(log_level=log_level)
     project_options = _get_project_options(args)
 
+    logger.debug("Starting snapcraft {} from {}.".format(
+        snapcraft.__version__, os.path.dirname(__file__)))
+
     if args['strip']:
         logger.warning("DEPRECATED: use 'prime' instead of 'strip'")
         args['prime'] = True
@@ -243,6 +273,7 @@ def _get_command_from_arg(args):
         'login': snapcraft.login,
         'logout': snapcraft.logout,
         'list-plugins': _list_plugins,
+        'plugins': _list_plugins,
     }
     function = [k for k in functions if args[k]]
     if len(function) == 0:
@@ -261,7 +292,7 @@ def run(args, project_options):  # noqa
     elif args['clean']:
         _run_clean(args, project_options)
     elif args['cleanbuild']:
-        lifecycle.cleanbuild(project_options),
+        lifecycle.cleanbuild(project_options, remote=args['--remote']),
     elif _is_store_command(args):
         _run_store_command(args)
     elif args['tour']:
@@ -269,6 +300,8 @@ def run(args, project_options):  # noqa
     elif args['help']:
         snapcraft.topic_help(args['<topic>'] or args['<plugin>'],
                              args['--devel'], args['topics'])
+    elif args['enable-ci']:
+        enable_ci(args['<ci-system>'], args['--refresh'])
     elif args['update']:
         parts.update()
     elif args['define']:
@@ -292,16 +325,22 @@ def _run_clean(args, project_options):
 
 def _is_store_command(args):
     commands = (
-        'list-keys', 'keys', 'register-key', 'register', 'sign-build',
-        'upload', 'release', 'push', 'validate', 'gated')
+        'list-registered', 'registered', 'list-keys', 'keys', 'create-key',
+        'register-key', 'register', 'sign-build', 'upload', 'release',
+        'push', 'validate', 'gated', 'history', 'revisions',
+        'list-revisions', 'status', 'close')
     return any(args.get(command) for command in commands)
 
 
 # This function's complexity is correlated to the number of
 # commands, no point in checking that.
 def _run_store_command(args):  # noqa: C901
-    if args['list-keys'] or args['keys']:
+    if args['list-registered'] or args['registered']:
+        snapcraft.list_registered()
+    elif args['list-keys'] or args['keys']:
         snapcraft.list_keys()
+    elif args['create-key']:
+        snapcraft.create_key(args['<key-name>'])
     elif args['register-key']:
         snapcraft.register_key(args['<key-name>'])
     elif args['register']:
@@ -326,6 +365,16 @@ def _run_store_command(args):  # noqa: C901
                            key=args['--key-name'])
     elif args['gated']:
         snapcraft.gated(args['<snap-name>'])
+    elif args['status']:
+        snapcraft.status(
+            args['<snap-name>'], args['--series'], args['--arch'])
+    elif args['revisions'] or args['list-revisions'] or args['history']:
+        if args['history']:
+            deprecations.handle_deprecation_notice('dn4')
+        snapcraft.revisions(
+            args['<snap-name>'], args['--series'], args['--arch'])
+    elif args['close']:
+        snapcraft.close(args['<snap-name>'], args['<channel_names>'])
 
 
 if __name__ == '__main__':  # pragma: no cover
