@@ -1,6 +1,6 @@
 # -*- Mode:Python; indent-tabs-mode:nil; tab-width:4 -*-
 #
-# Copyright (C) 2015, 2016 Canonical Ltd
+# Copyright (C) 2015-2016 Canonical Ltd
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License version 3 as
@@ -28,24 +28,45 @@ Additionally, this plugin uses the following plugin-specific keywords:
     - source-space:
       (string)
       The source space containing Catkin packages. By default this is 'src'.
+    - rosdistro:
+      (string)
+      The ROS distro required by this system. Defaults to 'indigo'.
     - include-roscore:
       (boolean)
       Whether or not to include roscore with the part. Defaults to true.
+    - underlay:
+      (object)
+      Used to inform Snapcraft that this snap isn't standalone, and is actually
+      overlaying a workspace from another snap via content sharing. Made up of
+      two properties:
+      - build-path:
+        (string)
+        Build-time path to existing workspace to underlay the one being built,
+        for example '$SNAPCRAFT_STAGE/opt/ros/kinetic'.
+      - run-path:
+        (string)
+        Run-time path of the underlay workspace (e.g. a subdirectory of the
+        content interface's 'target' attribute.)
 """
 
+import contextlib
+import glob
 import os
 import tempfile
 import logging
-import shutil
 import re
+import shutil
 import subprocess
+import textwrap
 
 import snapcraft
 from snapcraft import (
     common,
     file_utils,
+    formatting_utils,
     repo,
 )
+from snapcraft.internal import errors
 
 logger = logging.getLogger(__name__)
 
@@ -89,20 +110,34 @@ class CatkinPlugin(snapcraft.BasePlugin):
             'default': 'true',
         }
 
+        schema['properties']['underlay'] = {
+            'type': 'object',
+            'properties': {
+                'build-path': {
+                    'type': 'string',
+                },
+                'run-path': {
+                    'type': 'string',
+                }
+            },
+            'required': ['build-path', 'run-path'],
+        }
+
         schema['required'].append('catkin-packages')
 
+        return schema
+
+    @classmethod
+    def get_pull_properties(cls):
         # Inform Snapcraft of the properties associated with pulling. If these
         # change in the YAML Snapcraft will consider the pull step dirty.
-        schema['pull-properties'].extend(
-            ['rosdistro', 'catkin-packages', 'source-space',
-             'include-roscore'])
-
-        return schema
+        return ['rosdistro', 'catkin-packages', 'source-space',
+                'include-roscore', 'underlay']
 
     @property
     def PLUGIN_STAGE_SOURCES(self):
         return """
-deb http://packages.ros.org/ros/${{suffix}}/ {0} main
+deb http://packages.ros.org/ros/ubuntu/ {0} main
 deb http://${{prefix}}.ubuntu.com/${{suffix}}/ {0} main universe
 deb http://${{prefix}}.ubuntu.com/${{suffix}}/ {0}-updates main universe
 deb http://${{prefix}}.ubuntu.com/${{suffix}}/ {0}-security main universe
@@ -111,11 +146,18 @@ deb http://${{security}}.ubuntu.com/${{suffix}} {0}-security main universe
 
     def __init__(self, name, options, project):
         super().__init__(name, options, project)
-        self.build_packages.extend(['gcc', 'libc6-dev', 'make'])
+        self.build_packages.extend(['libc6-dev', 'make'])
+
+        # roslib is the base requiremet to actually create a workspace with
+        # setup.sh and the necessary hooks.
+        self.stage_packages.append(
+            'ros-{}-roslib'.format(self.options.rosdistro))
 
         # Get a unique set of packages
         self.catkin_packages = set(options.catkin_packages)
         self._rosdep_path = os.path.join(self.partdir, 'rosdep')
+        self._compilers_path = os.path.join(self.partdir, 'compilers')
+        self._catkin_path = os.path.join(self.partdir, 'catkin')
 
         # The path created via the `source` key (or a combination of `source`
         # and `source-subdir` keys) needs to point to a valid Catkin workspace
@@ -137,6 +179,15 @@ deb http://${{security}}.ubuntu.com/${{suffix}} {0}-security main universe
             raise RuntimeError(
                 'source-space cannot be the root of the Catkin workspace')
 
+        # Validate selected ROS distro
+        if self.options.rosdistro not in _ROS_RELEASE_MAP:
+            raise RuntimeError(
+                'Unsupported rosdistro: {!r}. The supported ROS distributions '
+                'are {}'.format(
+                    self.options.rosdistro,
+                    formatting_utils.humanize_list(
+                        _ROS_RELEASE_MAP.keys(), 'and')))
+
     def env(self, root):
         """Runtime environment for ROS binaries and services."""
 
@@ -154,12 +205,6 @@ deb http://${{security}}.ubuntu.com/${{suffix}} {0}-security main universe
             # temporarily work around that bug by forcing the locale to
             # C.UTF-8.
             'LC_ALL=C.UTF-8',
-
-            # This environment variable points to where the setup.sh and
-            # _setup_util.py files are located. This is required at both build-
-            # and run-time.
-            '_CATKIN_SETUP_DIR={}'.format(os.path.join(
-                root, 'opt', 'ros', self.options.rosdistro)),
         ]
 
         # There's a chicken and egg problem here, everything run get's an
@@ -178,13 +223,17 @@ deb http://${{security}}.ubuntu.com/${{suffix}} {0}-security main universe
         # make sure it's in the PATH before it's run.
         env.append('PATH=$PATH:{}/usr/bin'.format(root))
 
-        # FIXME: Nasty hack to source ROS's setup.sh (since each of these
-        # lines is prepended with "export"). There's got to be a better way
-        # to do this.
-        env.append(
-            'echo FOO=BAR\nif `test -e {0}` ; then\n. {0} ;\nfi\n'.format(
-                os.path.join(
-                    root, 'opt', 'ros', self.options.rosdistro, 'setup.sh')))
+        if self.options.underlay:
+            script = '. {}'.format(os.path.join(
+                self.rosdir, 'snapcraft-setup.sh'))
+        else:
+            script = self._source_setup_sh(root, None)
+
+        # Each of these lines is prepended with an `export` when the
+        # environment is actually generated. In order to inject real shell code
+        # we have to hack it in by appending it on the end of an item already
+        # in the environment. FIXME: There should be a better way to do this.
+        env[-1] = env[-1] + '\n\n' + script
 
         return env
 
@@ -201,20 +250,57 @@ deb http://${{security}}.ubuntu.com/${{suffix}} {0}-security main universe
         super().pull()
 
         # Make sure the package path exists before continuing
-        if not os.path.exists(self._ros_package_path):
+        if self.catkin_packages and not os.path.exists(self._ros_package_path):
             raise FileNotFoundError(
                 'Unable to find package path: "{}"'.format(
                     self._ros_package_path))
 
+        # Validate the underlay. Note that this validation can't happen in
+        # __init__ as the underlay will probably only be valid once a
+        # dependency has been staged.
+        catkin = None
+        underlay_build_path = None
+        if self.options.underlay:
+            underlay_build_path = self.options.underlay['build-path']
+        if underlay_build_path:
+            if not os.path.isdir(underlay_build_path):
+                raise errors.SnapcraftEnvironmentError(
+                    'Requested underlay ({!r}) does not point to a valid '
+                    'directory'.format(underlay_build_path))
+
+            if not os.path.isfile(os.path.join(underlay_build_path,
+                                               'setup.sh')):
+                raise errors.SnapcraftEnvironmentError(
+                    'Requested underlay ({!r}) does not contain a '
+                    'setup.sh'.format(underlay_build_path))
+
+            # Use catkin_find to discover dependencies already in the underlay
+            catkin = _Catkin(
+                self.options.rosdistro, underlay_build_path, self._catkin_path,
+                self.PLUGIN_STAGE_SOURCES, self.project)
+            catkin.setup()
+
+            self._generate_snapcraft_setup_sh(
+                self.installdir, underlay_build_path)
+
+        # Pull our own compilers so we use ones that match up with the version
+        # of ROS we're using.
+        compilers = _Compilers(
+            self._compilers_path, self.PLUGIN_STAGE_SOURCES, self.project)
+        compilers.setup()
+
         # Use rosdep for dependency detection and resolution
         rosdep = _Rosdep(self.options.rosdistro, self._ros_package_path,
                          self._rosdep_path, self.PLUGIN_STAGE_SOURCES,
                          self.project)
         rosdep.setup()
 
+        self._setup_dependencies(rosdep, catkin)
+
+    def _setup_dependencies(self, rosdep, catkin):
         # Parse the Catkin packages to pull out their system dependencies
-        system_dependencies = _find_system_dependencies(self.catkin_packages,
-                                                        rosdep)
+        system_dependencies = _find_system_dependencies(
+            self.catkin_packages, rosdep, catkin)
 
         # If the package requires roscore, resolve it into a system dependency
         # as well.
@@ -232,15 +318,14 @@ deb http://${{security}}.ubuntu.com/${{suffix}} {0}-security main universe
             os.makedirs(ubuntudir, exist_ok=True)
 
             logger.info('Preparing to fetch package dependencies...')
-            ubuntu = repo.Ubuntu(
-                ubuntudir, self.project,
-                sources=self.PLUGIN_STAGE_SOURCES,
-                project_options=self.project)
+            ubuntu = repo.Ubuntu(ubuntudir,
+                                 sources=self.PLUGIN_STAGE_SOURCES,
+                                 project_options=self.project)
 
             logger.info('Fetching package dependencies...')
             try:
                 ubuntu.get(system_dependencies)
-            except repo.PackageNotFoundError as e:
+            except repo.errors.PackageNotFoundError as e:
                 raise RuntimeError(
                     'Failed to fetch system dependencies: {}'.format(
                         e.message))
@@ -252,25 +337,83 @@ deb http://${{security}}.ubuntu.com/${{suffix}} {0}-security main universe
         super().clean_pull()
 
         # Remove the rosdep path, if any
-        if os.path.exists(self._rosdep_path):
+        with contextlib.suppress(FileNotFoundError):
             shutil.rmtree(self._rosdep_path)
 
-    @property
-    def gcc_version(self):
-        return self.run_output(['gcc', '-dumpversion'])
+        # Remove the compilers path, if any
+        with contextlib.suppress(FileNotFoundError):
+            shutil.rmtree(self._compilers_path)
+
+        # Remove the catkin path, if any
+        with contextlib.suppress(FileNotFoundError):
+            shutil.rmtree(self._catkin_path)
+
+    def _source_setup_sh(self, root, underlay_path):
+        rosdir = os.path.join(root, 'opt', 'ros', self.options.rosdistro)
+        if underlay_path:
+            source_script = textwrap.dedent('''
+                if [ -f {underlay_setup} ]; then
+                    _CATKIN_SETUP_DIR={underlay} . {underlay_setup}
+                    if [ -f {rosdir_setup} ]; then
+                        set -- --extend
+                        _CATKIN_SETUP_DIR={rosdir} . {rosdir_setup}
+                    fi
+                fi
+            ''').format(
+                underlay=underlay_path,
+                underlay_setup=os.path.join(underlay_path, 'setup.sh'),
+                rosdir=rosdir,
+                rosdir_setup=os.path.join(rosdir, 'setup.sh'))
+        else:
+            source_script = textwrap.dedent('''
+                if [ -f {rosdir_setup} ]; then
+                    _CATKIN_SETUP_DIR={rosdir} . {rosdir_setup}
+                fi
+            ''').format(
+                rosdir=rosdir,
+                rosdir_setup=os.path.join(rosdir, 'setup.sh'))
+
+        # We need to source ROS's setup.sh at this point. However, it accepts
+        # arguments (thus will parse $@), and we really don't want it to, since
+        # $@ in this context will be meant for the app being launched
+        # (LP: #1660852). So we'll backup all args, source the setup.sh, then
+        # restore all args for the wrapper's `exec` line.
+        return textwrap.dedent('''
+            # Shell quote arbitrary string by replacing every occurrence of '
+            # with '\\'', then put ' at the beginning and end of the string.
+            # Prepare yourself, fun regex ahead.
+            quote()
+            {{
+                for i; do
+                    printf %s\\\\n "$i" | sed "s/\'/\'\\\\\\\\\'\'/g;1s/^/\'/;\$s/\$/\' \\\\\\\\/"
+                done
+                echo " "
+            }}
+
+            BACKUP_ARGS=$(quote "$@")
+            set --
+            {}
+            eval "set -- $BACKUP_ARGS"
+        ''').format(source_script)  # noqa
+
+    def _generate_snapcraft_setup_sh(self, root, underlay_path):
+        script = self._source_setup_sh(root, underlay_path)
+        os.makedirs(self.rosdir, exist_ok=True)
+        with open(os.path.join(self.rosdir, 'snapcraft-setup.sh'), 'w') as f:
+            f.write(script)
 
     @property
     def rosdir(self):
         return os.path.join(self.installdir, 'opt', 'ros',
                             self.options.rosdistro)
 
-    def _run_in_bash(self, commandlist, cwd=None):
+    def _run_in_bash(self, commandlist, cwd=None, env=None):
         with tempfile.NamedTemporaryFile(mode='w') as f:
-            f.write('set -ex\n')
+            f.write('set -e\n')
             f.write('exec {}\n'.format(' '.join(commandlist)))
             f.flush()
 
-            self.run(['/bin/bash', f.name], cwd=cwd)
+            self.run(['/bin/bash', f.name], cwd=cwd, env=env)
 
     def build(self):
         """Build Catkin packages.
@@ -300,36 +443,70 @@ deb http://${{security}}.ubuntu.com/${{suffix}} {0}-security main universe
         # dependencies contain .cmake files pointing to system paths (e.g.
         # /usr/lib, /usr/include, etc.). They need to be rewritten to point to
         # the install directory.
-        def rewrite_paths(match):
+        def _new_path(path):
+            if not path.startswith(self.installdir):
+                # Not using os.path.join here as `path` is absolute.
+                return self.installdir + path
+            return path
+
+        self._rewrite_cmake_paths(_new_path)
+
+    def _rewrite_cmake_paths(self, new_path_callable):
+        def _rewrite_paths(match):
             paths = match.group(1).strip().split(';')
             for i, path in enumerate(paths):
-                # Rewrite this path if it's an absolute path and not already
-                # within the install directory.
-                if (os.path.isabs(path) and
-                        not path.startswith(self.installdir)):
-                    paths[i] = self.installdir + path
+                # Offer the opportunity to rewrite this path if it's absolute.
+                if os.path.isabs(path):
+                    paths[i] = new_path_callable(path)
 
             return '"' + ';'.join(paths) + '"'
 
         # Looking for any path-like string
         file_utils.replace_in_file(self.rosdir, re.compile(r'.*Config.cmake$'),
                                    re.compile(r'"(.*?/.*?)"'),
-                                   rewrite_paths)
+                                   _rewrite_paths)
 
     def _finish_build(self):
         self._use_in_snap_python()
 
+        # We've finished the build, but we need to make sure we turn the cmake
+        # files back into something that doesn't include our installdir. This
+        # way it's usable from the staging area, and won't clash with the same
+        # file coming from other parts.
+        pattern = re.compile(r'^{}'.format(self.installdir))
+
+        def _new_path(path):
+            return pattern.sub('$ENV{SNAPCRAFT_STAGE}', path)
+        self._rewrite_cmake_paths(_new_path)
+
         # Replace the CMAKE_PREFIX_PATH in _setup_util.sh
         setup_util_file = os.path.join(self.rosdir, '_setup_util.py')
         if os.path.isfile(setup_util_file):
             with open(setup_util_file, 'r+') as f:
-                pattern = re.compile(r"CMAKE_PREFIX_PATH = '{}.*".format(
-                    self.rosdir))
+                pattern = re.compile(r"CMAKE_PREFIX_PATH = '.*/opt/ros.*")
                 replaced = pattern.sub('CMAKE_PREFIX_PATH = []', f.read())
                 f.seek(0)
                 f.truncate()
                 f.write(replaced)
 
+        # Set the _CATKIN_SETUP_DIR in setup.sh to a sensible default, removing
+        # our installdir (this way it doesn't clash with a setup.sh coming
+        # from another part).
+        setup_sh_file = os.path.join(self.rosdir, 'setup.sh')
+        if os.path.isfile(setup_sh_file):
+            with open(setup_sh_file, 'r+') as f:
+                pattern = re.compile(r"\${_CATKIN_SETUP_DIR:=.*}")
+                replaced = pattern.sub(
+                    '${{_CATKIN_SETUP_DIR:=$SNAP/opt/ros/{}}}'.format(
+                        self.options.rosdistro), f.read())
+                f.seek(0)
+                f.truncate()
+                f.write(replaced)
+
+        if self.options.underlay:
+            underlay_run_path = self.options.underlay['run-path']
+            self._generate_snapcraft_setup_sh('$SNAP', underlay_run_path)
+
     def _use_in_snap_python(self):
         # Fix all shebangs to use the in-snap python.
         file_utils.replace_in_file(self.rosdir, re.compile(r''),
@@ -376,30 +553,38 @@ deb http://${{security}}.ubuntu.com/${{suffix}} {0}-security main universe
         # All the arguments that follow are meant for CMake
         catkincmd.append('--cmake-args')
 
-        # Make sure we're using the compilers included in this .snap
+        # Make sure we're using our own compilers (the one on the system may
+        # be the wrong version).
+        compilers = _Compilers(
+            self._compilers_path, self.PLUGIN_STAGE_SOURCES, self.project)
         catkincmd.extend([
-            '-DCMAKE_C_FLAGS="$CFLAGS"',
-            '-DCMAKE_CXX_FLAGS="$CPPFLAGS -I{} -I{}"'.format(
-                os.path.join(self.installdir, 'usr', 'include', 'c++',
-                             self.gcc_version),
-                os.path.join(self.installdir, 'usr', 'include',
-                             self.project.arch_triplet, 'c++',
-                             self.gcc_version)),
-            '-DCMAKE_LD_FLAGS="$LDFLAGS"',
-            '-DCMAKE_C_COMPILER={}'.format(
-                os.path.join(self.installdir, 'usr', 'bin', 'gcc')),
-            '-DCMAKE_CXX_COMPILER={}'.format(
-                os.path.join(self.installdir, 'usr', 'bin', 'g++'))
+            '-DCMAKE_C_FLAGS="$CFLAGS {}"'.format(compilers.cflags),
+            '-DCMAKE_CXX_FLAGS="$CPPFLAGS {}"'.format(compilers.cxxflags),
+            '-DCMAKE_LD_FLAGS="$LDFLAGS {}"'.format(compilers.ldflags),
+            '-DCMAKE_C_COMPILER={}'.format(compilers.c_compiler_path),
+            '-DCMAKE_CXX_COMPILER={}'.format(compilers.cxx_compiler_path)
         ])
 
         # This command must run in bash due to a bug in Catkin that causes it
         # to explode if there are spaces in the cmake args (which there are).
         # This has been fixed in Catkin Tools... perhaps we should be using
         # that instead.
-        self._run_in_bash(catkincmd)
+        self._run_in_bash(catkincmd, env=compilers.environment)
+
+    def snap_fileset(self):
+        """Filter useless files out of the snap.
+
+        - opt/ros/<rosdistro>/.rosinstall points to the part installdir, and
+          isn't useful from the snap anyway.
+        """
+
+        fileset = super().snap_fileset()
+        fileset.append('-{}'.format(
+            os.path.join('opt', 'ros', self.options.rosdistro, '.rosinstall')))
+        return fileset
 
 
-def _find_system_dependencies(catkin_packages, rosdep):
+def _find_system_dependencies(catkin_packages, rosdep, catkin):
     """Find system dependencies for a given set of Catkin packages."""
 
     system_dependencies = {}
@@ -416,13 +601,29 @@ def _find_system_dependencies(catkin_packages, rosdep):
                     dependency in system_dependencies):
                 continue
 
+            if catkin:
+                # Before trying to resolve this dependency into a system
+                # dependency, see if it's already in the underlay.
+                try:
+                    catkin.find(dependency)
+                except CatkinPackageNotFoundError:
+                    # No package by that name is available
+                    pass
+                else:
+                    # Package was found-- don't pull anything extra to satisfy
+                    # this dependency.
+                    logger.debug(
+                        'Satisfied dependency {!r} in underlay'.format(
+                            dependency))
+                    continue
+
             # In this situation, the package depends on something that we
             # weren't instructed to build. It's probably a system dependency,
             # but the developer could have also forgotten to tell us to build
             # it.
             try:
                 these_dependencies = rosdep.resolve_dependency(dependency)
-            except SystemDependencyNotFound:
+            except SystemDependencyNotFoundError:
                 raise RuntimeError(
                     "Package {!r} isn't a valid system dependency. "
                     "Did you forget to add it to catkin-packages? If "
@@ -432,19 +633,23 @@ def _find_system_dependencies(catkin_packages, rosdep):
 
             system_dependencies[dependency] = these_dependencies
 
-            # TODO: Not sure why this isn't pulled in by roscpp. Can it
-            # be compiled by clang, etc.? If so, perhaps this should be
-            # left up to the developer.
-            if dependency == 'roscpp':
-                system_dependencies['g++'] = ['g++']
-
     # Finally, return a list of all system dependencies
     return set(item for sublist in system_dependencies.values()
                for item in sublist)
 
 
-class SystemDependencyNotFound(Exception):
-    pass
+class SystemDependencyNotFoundError(errors.SnapcraftError):
+    fmt = '{system_dependency!r} does not resolve to a system dependency'
+
+    def __init__(self, system_dependency):
+        super().__init__(system_dependency=system_dependency)
+
+
+class CatkinPackageNotFoundError(errors.SnapcraftError):
+    fmt = 'Unable to find Catkin package {package_name!r}'
+
+    def __init__(self, package_name):
+        super().__init__(package_name=package_name)
 
 
 class _Rosdep:
@@ -525,9 +730,7 @@ class _Rosdep:
                                 'ubuntu:{}'.format(
                                     _ROS_RELEASE_MAP[self._ros_distro])])
         except subprocess.CalledProcessError:
-            raise SystemDependencyNotFound(
-                '{!r} does not resolve to a system dependency'.format(
-                    dependency_name))
+            raise SystemDependencyNotFoundError(dependency_name)
 
         # Everything that isn't a package name is prepended with the pound
         # sign, so we'll ignore everything with that.
@@ -559,3 +762,150 @@ class _Rosdep:
 
         return subprocess.check_output(['rosdep'] + arguments,
                                        env=env).decode('utf8').strip()
+
+
+class _Compilers:
+    def __init__(self, compilers_path, ubuntu_sources, project):
+        self._compilers_path = compilers_path
+        self._ubuntu_sources = ubuntu_sources
+        self._project = project
+
+        self._compilers_install_path = os.path.join(
+            self._compilers_path, 'install')
+        self.__gcc_version = None
+
+    def setup(self):
+        os.makedirs(self._compilers_install_path, exist_ok=True)
+
+        # Since we support building older ROS distros we need to make sure we
+        # use the corresponding compiler versions, so they can't be
+        # build-packages. We'll just download them to another place and use
+        # them from there.
+        logger.info('Preparing to fetch compilers...')
+        ubuntu = repo.Ubuntu(
+            self._compilers_path, sources=self._ubuntu_sources,
+            project_options=self._project)
+
+        logger.info('Fetching compilers...')
+        ubuntu.get(['gcc', 'g++'])
+
+        logger.info('Installing compilers...')
+        ubuntu.unpack(self._compilers_install_path)
+
+    @property
+    def environment(self):
+        env = os.environ.copy()
+
+        paths = common.get_library_paths(
+            self._compilers_install_path, self._project.arch_triplet)
+        ld_library_path = formatting_utils.combine_paths(
+            paths, prepend='', separator=':')
+
+        env['LD_LIBRARY_PATH'] = (
+            env.get('LD_LIBRARY_PATH', '') + ':' + ld_library_path)
+
+        env['PATH'] = env.get('PATH', '') + ':' + os.path.join(
+            self._compilers_install_path, 'usr', 'bin')
+
+        return env
+
+    @property
+    def c_compiler_path(self):
+        return os.path.join(self._compilers_install_path, 'usr', 'bin', 'gcc')
+
+    @property
+    def cxx_compiler_path(self):
+        return os.path.join(self._compilers_install_path, 'usr', 'bin', 'g++')
+
+    @property
+    def cflags(self):
+        return ''
+
+    @property
+    def cxxflags(self):
+        paths = set(common.get_include_paths(
+            self._compilers_install_path, self._project.arch_triplet))
+
+        try:
+            paths.add(_get_highest_version_path(os.path.join(
+                self._compilers_install_path, 'usr', 'include', 'c++')))
+            paths.add(_get_highest_version_path(os.path.join(
+                self._compilers_install_path, 'usr', 'include',
+                self._project.arch_triplet, 'c++')))
+        except RuntimeError as e:
+            raise RuntimeError('Unable to determine gcc version: {}'.format(
+                str(e)))
+
+        return formatting_utils.combine_paths(
+            paths, prepend='-I', separator=' ')
+
+    @property
+    def ldflags(self):
+        paths = common.get_library_paths(
+            self._compilers_install_path, self._project.arch_triplet)
+        return formatting_utils.combine_paths(
+            paths, prepend='-L', separator=' ')
+
+
+class _Catkin:
+    def __init__(self, ros_distro, workspace, catkin_path, ubuntu_sources,
+                 project):
+        self._ros_distro = ros_distro
+        self._workspace = workspace
+        self._catkin_path = catkin_path
+        self._ubuntu_sources = ubuntu_sources
+        self._project = project
+        self._catkin_install_path = os.path.join(self._catkin_path, 'install')
+
+    def setup(self):
+        os.makedirs(self._catkin_install_path, exist_ok=True)
+
+        # With the introduction of an underlay, we no longer know where Catkin
+        # is. Let's just fetch/unpack our own, and use it.
+        logger.info('Preparing to fetch catkin...')
+        ubuntu = repo.Ubuntu(self._catkin_path, sources=self._ubuntu_sources,
+                             project_options=self._project)
+        logger.info('Fetching catkin...')
+        ubuntu.get(['ros-{}-catkin'.format(self._ros_distro)])
+
+        logger.info('Installing catkin...')
+        ubuntu.unpack(self._catkin_install_path)
+
+    def find(self, package_name):
+        try:
+            return self._run(['--first-only', package_name]).strip()
+        except subprocess.CalledProcessError:
+            raise CatkinPackageNotFoundError(package_name)
+
+    def _run(self, arguments):
+        with tempfile.NamedTemporaryFile(mode='w+') as f:
+            lines = ['export PYTHONPATH={}'.format(os.path.join(
+                self._catkin_install_path, 'usr', 'lib', 'python2.7',
+                'dist-packages'))]
+
+            ros_path = os.path.join(
+                self._catkin_install_path, 'opt', 'ros', self._ros_distro)
+            bin_paths = (
+                os.path.join(ros_path, 'bin'),
+                os.path.join(self._catkin_install_path, 'usr', 'bin'))
+            lines.append('export {}'.format(
+                formatting_utils.format_path_variable(
+                    'PATH', bin_paths, prepend='', separator=':')))
+
+            lines.append('export _CATKIN_SETUP_DIR={}'.format(self._workspace))
+            lines.append('source {}'.format(os.path.join(
+                self._workspace, 'setup.sh')))
+            lines.append('exec "$@"')
+            f.write('\n'.join(lines))
+            f.flush()
+            return subprocess.check_output(
+                ['/bin/bash', f.name, 'catkin_find'] + arguments,
+                stderr=subprocess.STDOUT).decode('utf8').strip()
+
+
+def _get_highest_version_path(path):
+    paths = sorted(glob.glob(os.path.join(path, '*')))
+    if not paths:
+        raise RuntimeError('nothing found in {!r}'.format(path))
+
+    return paths[-1]
