@@ -23,7 +23,7 @@ WARNING: this plugin's API is unstable. The cross compiling support is
 The following kernel specific options are provided by this plugin:
 
     - kernel-image-target:
-      (yaml object or string; default: bzImage)
+      (yaml object, string or null for default target)
       the default target is bzImage and can be set to any specific
       target.
       For more complex cases where one would want to use
@@ -74,6 +74,13 @@ _compression_command = {
     'gz': 'gzip',
 }
 
+default_kernel_image_target = {
+    'amd64': 'bzImage',
+    'i386': 'bzImage',
+    'armhf': 'zImage',
+    'arm64': 'Image.gz',
+}
+
 
 class KernelPlugin(kbuild.KBuildPlugin):
 
@@ -86,7 +93,7 @@ class KernelPlugin(kbuild.KBuildPlugin):
                 {'type': 'string'},
                 {'type': 'object'},
             ],
-            'default': 'bzImage',
+            'default': '',
         }
 
         schema['properties']['kernel-with-firmware'] = {
@@ -130,14 +137,16 @@ class KernelPlugin(kbuild.KBuildPlugin):
             'enum': ['gz'],
         }
 
+        return schema
+
+    @classmethod
+    def get_build_properties(cls):
         # Inform Snapcraft of the properties associated with building. If these
         # change in the YAML Snapcraft will consider the build step dirty.
-        schema['build-properties'].extend([
+        return super().get_build_properties() + [
             'kernel-image-target', 'kernel-with-firmware',
             'kernel-initrd-modules', 'kernel-initrd-firmware',
-            'kernel-device-trees', 'kernel-initrd-compression'])
-
-        return schema
+            'kernel-device-trees', 'kernel-initrd-compression']
 
     @property
     def compression_cmd(self):
@@ -164,19 +173,28 @@ class KernelPlugin(kbuild.KBuildPlugin):
         self._set_kernel_targets()
 
     def _set_kernel_targets(self):
-        if isinstance(self.options.kernel_image_target, str):
+        if not self.options.kernel_image_target:
+            self.kernel_image_target = \
+                default_kernel_image_target[self.project.deb_arch]
+        elif isinstance(self.options.kernel_image_target, str):
             self.kernel_image_target = self.options.kernel_image_target
         elif self.project.deb_arch in self.options.kernel_image_target:
             self.kernel_image_target = \
                 self.options.kernel_image_target[self.project.deb_arch]
 
         self.make_targets = [self.kernel_image_target, 'modules']
+        self.make_install_targets = [
+            'modules_install', 'INSTALL_MOD_PATH={}'.format(self.installdir)]
         self.dtbs = ['{}.dtb'.format(i)
                      for i in self.options.kernel_device_trees]
         if self.dtbs:
             self.make_targets.extend(self.dtbs)
-        self.make_install_targets = [
-            'modules_install', 'INSTALL_MOD_PATH={}'.format(self.installdir)]
+        elif (self.project.kernel_arch == "arm" or
+                self.project.kernel_arch == "arm64"):
+            self.make_targets.append('dtbs')
+            self.make_install_targets.extend([
+                'dtbs_install',
+                'INSTALL_DTBS_PATH={}/dtbs'.format(self.installdir)])
         self.make_install_targets.extend(self._get_fw_install_targets())
 
     def _get_fw_install_targets(self):
@@ -236,6 +254,7 @@ class KernelPlugin(kbuild.KBuildPlugin):
         return initrd_unpacked_path
 
     def _make_initrd(self):
+
         logger.info('Generating driver initrd for kernel release: {}'.format(
             self.kernel_release))
 
@@ -248,9 +267,9 @@ class KernelPlugin(kbuild.KBuildPlugin):
                 '-S', self.kernel_release, module])
             modprobe_outs.extend(modprobe_out.split(os.linesep))
 
+        modprobe_outs = [_ for _ in modprobe_outs if _]
         modules_path = os.path.join('lib', 'modules', self.kernel_release)
-        for src in set(modprobe_outs):
-            src = src.split()[-1:][0]
+        for src in set(_.split()[1] for _ in modprobe_outs):
             dst = os.path.join(initrd_unpacked_path,
                                os.path.relpath(src, self.installdir))
             os.makedirs(os.path.dirname(dst), exist_ok=True)
@@ -349,3 +368,11 @@ class KernelPlugin(kbuild.KBuildPlugin):
         self._copy_vmlinuz()
         self._copy_system_map()
         self._copy_dtbs()
+
+        # upstream kernel installs under $INSTALL_MOD_PATH/lib/modules/
+        # but snapd expects modules/ and firmware/
+        shutil.move(
+            os.path.join(self.installdir, 'lib', 'modules'), self.installdir)
+        shutil.move(
+            os.path.join(self.installdir, 'lib', 'firmware'), self.installdir)
+        os.rmdir(os.path.join(self.installdir, 'lib'))
