@@ -39,7 +39,6 @@ Additionally, this plugin uses the following plugin-specific keywords:
 
 import logging
 import os
-import platform
 import shutil
 
 import snapcraft
@@ -51,10 +50,10 @@ _NODEJS_BASE = 'node-v{version}-linux-{arch}'
 _NODEJS_VERSION = '4.4.4'
 _NODEJS_TMPL = 'https://nodejs.org/dist/v{version}/{base}.tar.gz'
 _NODEJS_ARCHES = {
-    'i686': 'x86',
-    'x86_64': 'x64',
-    'armv7l': 'armv7l',
-    'aarch64': 'arm64',
+    'i386': 'x86',
+    'amd64': 'x64',
+    'armhf': 'armv7l',
+    'arm64': 'arm64',
 }
 
 
@@ -90,27 +89,32 @@ class NodePlugin(snapcraft.BasePlugin):
         if 'required' in schema:
             del schema['required']
 
+        return schema
+
+    @classmethod
+    def get_build_properties(cls):
         # Inform Snapcraft of the properties associated with building. If these
         # change in the YAML Snapcraft will consider the build step dirty.
-        schema['build-properties'].extend(['node-packages', 'npm-run'])
+        return ['node-packages', 'npm-run']
+
+    @classmethod
+    def get_pull_properties(cls):
         # Inform Snapcraft of the properties associated with pulling. If these
         # change in the YAML Snapcraft will consider the build step dirty.
-        schema['pull-properties'].append('node-engine')
-
-        return schema
+        return ['node-engine']
 
     def __init__(self, name, options, project):
         super().__init__(name, options, project)
         self._npm_dir = os.path.join(self.partdir, 'npm')
         self._nodejs_tar = sources.Tar(get_nodejs_release(
-            self.options.node_engine), self._npm_dir)
+            self.options.node_engine, self.project.deb_arch), self._npm_dir)
 
     def pull(self):
         super().pull()
         os.makedirs(self._npm_dir, exist_ok=True)
         self._nodejs_tar.download()
         # do the install in the pull phase to download all dependencies.
-        self._npm_install()
+        self._npm_install(rootdir=self.sourcedir)
 
     def clean_pull(self):
         super().clean_pull()
@@ -121,23 +125,22 @@ class NodePlugin(snapcraft.BasePlugin):
 
     def build(self):
         super().build()
-        self._npm_install()
+        self._npm_install(rootdir=self.builddir)
 
-    def _npm_install(self):
+    def _npm_install(self, rootdir):
         self._nodejs_tar.provision(
             self.installdir, clean_target=False, keep_tarball=True)
         npm_install = ['npm', '--cache-min=Infinity', 'install']
         for pkg in self.options.node_packages:
-            self.run(npm_install + ['--global'] + [pkg])
-        if os.path.exists(os.path.join(self.builddir, 'package.json')):
-            self.run(npm_install)
-            self.run(npm_install + ['--global'])
+            self.run(npm_install + ['--global'] + [pkg], cwd=rootdir)
+        if os.path.exists(os.path.join(rootdir, 'package.json')):
+            self.run(npm_install, cwd=rootdir)
+            self.run(npm_install + ['--global'], cwd=rootdir)
         for target in self.options.npm_run:
-            self.run(['npm', 'run', target])
+            self.run(['npm', 'run', target], cwd=rootdir)
 
 
-def _get_nodejs_base(node_engine):
-    machine = platform.machine()
+def _get_nodejs_base(node_engine, machine):
     if machine not in _NODEJS_ARCHES:
         raise EnvironmentError('architecture not supported ({})'.format(
             machine))
@@ -145,6 +148,6 @@ def _get_nodejs_base(node_engine):
                                arch=_NODEJS_ARCHES[machine])
 
 
-def get_nodejs_release(node_engine):
+def get_nodejs_release(node_engine, arch):
     return _NODEJS_TMPL.format(version=node_engine,
-                               base=_get_nodejs_base(node_engine))
+                               base=_get_nodejs_base(node_engine, arch))
