@@ -1,6 +1,7 @@
+
 # -*- Mode:Python; indent-tabs-mode:nil; tab-width:4 -*-
 #
-# Copyright (C) 2016 Canonical Ltd
+# Copyright (C) 2016-2017 Canonical Ltd
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License version 3 as
@@ -14,6 +15,7 @@
 # You should have received a copy of the GNU General Public License
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
+import contextlib
 import hashlib
 import itertools
 import json
@@ -31,12 +33,14 @@ from progressbar import (
 )
 import pymacaroons
 import requests
+from requests.adapters import HTTPAdapter
 from simplejson.scanner import JSONDecodeError
 
 import snapcraft
 from snapcraft import config
 from snapcraft.internal.indicators import download_requests_stream
 from snapcraft.storeapi import (
+    _agent,
     _upload,
     constants,
     errors,
@@ -89,11 +93,24 @@ class Client():
         self.conf = conf
         self.root_url = root_url
         self.session = requests.Session()
+        # Setup max retries for all store URLs and the CDN
+        self.session.mount('http://', HTTPAdapter(max_retries=5))
+        self.session.mount('https://', HTTPAdapter(max_retries=5))
+
+        self._snapcraft_headers = {
+            'User-Agent': _agent.get_user_agent(),
+        }
 
     def request(self, method, url, params=None, headers=None, **kwargs):
         """Overriding base class to handle the root url."""
         # Note that url may be absolute in which case 'root_url' is ignored by
         # urljoin.
+
+        if headers:
+            headers.update(self._snapcraft_headers)
+        else:
+            headers = self._snapcraft_headers
+
         final_url = urllib.parse.urljoin(self.root_url, url)
         response = self.session.request(
             method, final_url, headers=headers,
@@ -122,13 +139,13 @@ class StoreClient():
         self.sca = SCAClient(self.conf)
 
     def login(self, email, password, one_time_password=None, acls=None,
-              save=True):
+              packages=None, channels=None, save=True):
         """Log in via the Ubuntu One SSO API."""
         if acls is None:
             acls = ['package_upload', 'package_access']
         # Ask the store for the needed capabilities to be associated with the
         # macaroon.
-        macaroon = self.sca.get_macaroon(acls)
+        macaroon = self.sca.get_macaroon(acls, packages, channels)
         caveat_id = self._extract_caveat_id(macaroon)
         unbound_discharge = self.sso.get_unbound_discharge(
             email, password, one_time_password, caveat_id)
@@ -141,7 +158,7 @@ class StoreClient():
     def _extract_caveat_id(self, root_macaroon):
         macaroon = pymacaroons.Macaroon.deserialize(root_macaroon)
         # macaroons are all bytes, never strings
-        sso_host = urllib.parse.urlparse(self.sso.root_url).hostname
+        sso_host = urllib.parse.urlparse(self.sso.root_url).netloc
         for caveat in macaroon.caveats:
             if caveat.location == sso_host:
                 return caveat.caveat_id
@@ -174,11 +191,16 @@ class StoreClient():
         return self._refresh_if_necessary(
             self.sca.register, snap_name, is_private, constants.DEFAULT_SERIES)
 
+    def push_precheck(self, snap_name):
+        return self._refresh_if_necessary(
+            self.sca.snap_push_precheck, snap_name)
+
     def push_snap_build(self, snap_id, snap_build):
         return self._refresh_if_necessary(
             self.sca.push_snap_build, snap_id, snap_build)
 
-    def upload(self, snap_name, snap_filename):
+    def upload(self, snap_name, snap_filename, delta_format=None,
+               source_hash=None, target_hash=None, delta_hash=None):
         # FIXME This should be raised by the function that uses the
         # discharge. --elopio -2016-06-20
         if self.conf.get('unbound_discharge') is None:
@@ -188,20 +210,66 @@ class StoreClient():
         updown_data = _upload.upload_files(snap_filename, self.updown)
 
         return self._refresh_if_necessary(
-            self.sca.snap_push_metadata, snap_name, updown_data)
+            self.sca.snap_push_metadata, snap_name, updown_data,
+            delta_format=delta_format, source_hash=source_hash,
+            target_hash=target_hash, delta_hash=delta_hash)
 
     def release(self, snap_name, revision, channels):
         return self._refresh_if_necessary(
             self.sca.snap_release, snap_name, revision, channels)
 
-    def download(self, snap_name, channel, download_path, arch=None):
+    def get_snap_revisions(self, snap_name, series=None, arch=None):
+        if series is None:
+            series = constants.DEFAULT_SERIES
+
+        account_info = self.get_account_information()
+        try:
+            snap_id = account_info['snaps'][series][snap_name]['snap-id']
+        except KeyError:
+            raise errors.SnapNotFoundError(snap_name, series=series, arch=arch)
+
+        response = self._refresh_if_necessary(
+            self.sca.snap_revisions, snap_id, series, arch)
+
+        if not response:
+            raise errors.SnapNotFoundError(snap_name, series=series, arch=arch)
+
+        return response
+
+    def get_snap_status(self, snap_name, series=None, arch=None):
+        if series is None:
+            series = constants.DEFAULT_SERIES
+
+        account_info = self.get_account_information()
+        try:
+            snap_id = account_info['snaps'][series][snap_name]['snap-id']
+        except KeyError:
+            raise errors.SnapNotFoundError(snap_name, series=series, arch=arch)
+
+        response = self._refresh_if_necessary(
+            self.sca.snap_status, snap_id, series, arch)
+
+        if not response:
+            raise errors.SnapNotFoundError(snap_name, series=series, arch=arch)
+
+        return response
+
+    def close_channels(self, snap_id, channel_names):
+        return self._refresh_if_necessary(
+            self.sca.close_channels, snap_id, channel_names)
+
+    def download(self, snap_name, channel, download_path,
+                 arch=None, except_hash=''):
         if arch is None:
             arch = snapcraft.ProjectOptions().deb_arch
 
         package = self.cpi.get_package(snap_name, channel, arch)
-        self._download_snap(
-            snap_name, channel, arch, download_path,
-            package['download_url'], package['download_sha512'])
+        if package['download_sha3_384'] != except_hash:
+            self._download_snap(
+                snap_name, channel, arch, download_path,
+                # FIXME LP: #1662665
+                package['anon_download_url'], package['download_sha512'])
+        return package['download_sha3_384']
 
     def _download_snap(self, name, channel, arch, download_path,
                        download_url, expected_sha512):
@@ -237,6 +305,9 @@ class StoreClient():
     def get_validations(self, snap_id):
         return self.sca.get_validations(snap_id)
 
+    def sign_developer_agreement(self, latest_tos_accepted=False):
+        return self.sca.sign_developer_agreement(latest_tos_accepted)
+
 
 class SSOClient(Client):
     """The Single Sign On server deals with authentication.
@@ -301,17 +372,40 @@ class SnapIndexClient(Client):
             'UBUNTU_STORE_SEARCH_ROOT_URL',
             constants.UBUNTU_STORE_SEARCH_ROOT_URL))
 
+    def get_default_headers(self):
+        """Return default headers for CPI requests.
+
+        Tries to build an 'Authorization' header with local credentials
+        if they are available.
+        Also pin specific branded store if `SNAPCRAFT_UBUNTU_STORE`
+        environment is set.
+        """
+        headers = {}
+
+        with contextlib.suppress(errors.InvalidCredentialsError):
+            headers['Authorization'] = _macaroon_auth(self.conf)
+
+        branded_store = os.getenv('SNAPCRAFT_UBUNTU_STORE')
+        if branded_store:
+            headers['X-Ubuntu-Store'] = branded_store
+
+        return headers
+
     def get_package(self, snap_name, channel, arch=None):
-        headers = {
+        headers = self.get_default_headers()
+        headers.update({
             'Accept': 'application/hal+json',
             'X-Ubuntu-Release': constants.DEFAULT_SERIES,
-        }
+        })
         if arch:
             headers['X-Ubuntu-Architecture'] = arch
 
         params = {
             'channel': channel,
-            'fields': 'status,download_url,download_sha512,snap_id,release',
+            # FIXME LP: #1662665
+            'fields': 'status,anon_download_url,download_url,'
+                      'download_sha3_384,download_sha512,snap_id,'
+                      'revision,release',
         }
         logger.info('Getting details for {}'.format(snap_name))
         url = 'api/v1/snaps/details/{}'.format(snap_name)
@@ -322,8 +416,7 @@ class SnapIndexClient(Client):
 
     def get(self, url, headers=None, params=None, stream=False):
         if headers is None:
-            headers = {}
-        headers.update({'Authorization': _macaroon_auth(self.conf)})
+            headers = self.get_default_headers()
         response = self.request('GET', url, stream=stream,
                                 headers=headers, params=params)
         return response
@@ -352,12 +445,23 @@ class SCAClient(Client):
             'UBUNTU_STORE_API_ROOT_URL',
             constants.UBUNTU_STORE_API_ROOT_URL))
 
-    def get_macaroon(self, acls):
+    def get_macaroon(self, acls, packages=None, channels=None):
+        data = {
+            'permissions': acls,
+        }
+        if packages is not None:
+            data.update({
+                'packages': packages,
+            })
+        if channels is not None:
+            data.update({
+                'channels': channels,
+            })
+        headers = {
+            'Accept': 'application/json',
+        }
         response = self.post(
-            'acl/',
-            data=json.dumps({'permissions': acls}),
-            headers={'Content-Type': 'application/json',
-                     'Accept': 'application/json'})
+            'acl/', json=data, headers=headers)
         if response.ok:
             return response.json()['macaroon']
         else:
@@ -409,7 +513,23 @@ class SCAClient(Client):
         if not response.ok:
             raise errors.StoreRegistrationError(snap_name, response)
 
-    def snap_push_metadata(self, snap_name, updown_data):
+    def snap_push_precheck(self, snap_name):
+        data = {
+            'name': snap_name,
+            'dry_run': True,
+        }
+        auth = _macaroon_auth(self.conf)
+        response = self.post(
+            'snap-push/', data=json.dumps(data),
+            headers={'Authorization': auth,
+                     'Content-Type': 'application/json',
+                     'Accept': 'application/json'})
+        if not response.ok:
+            raise errors.StorePushError(data['name'], response)
+
+    def snap_push_metadata(self, snap_name, updown_data,
+                           delta_format=None, delta_hash=None,
+                           source_hash=None, target_hash=None):
         data = {
             'name': snap_name,
             'series': constants.DEFAULT_SERIES,
@@ -417,6 +537,11 @@ class SCAClient(Client):
             'binary_filesize': updown_data['binary_filesize'],
             'source_uploaded': updown_data['source_uploaded'],
         }
+        if delta_format:
+            data['delta_format'] = delta_format
+            data['delta_hash'] = delta_hash
+            data['source_hash'] = source_hash
+            data['target_hash'] = target_hash
         auth = _macaroon_auth(self.conf)
         response = self.post(
             'snap-push/', data=json.dumps(data),
@@ -428,12 +553,14 @@ class SCAClient(Client):
 
         return StatusTracker(response.json()['status_details_url'])
 
-    def snap_release(self, snap_name, revision, channels):
+    def snap_release(self, snap_name, revision, channels, delta_format=None):
         data = {
             'name': snap_name,
             'revision': str(revision),
             'channels': channels,
         }
+        if delta_format:
+            data['delta_format'] = delta_format
         auth = _macaroon_auth(self.conf)
         response = self.post(
             'snap-release/', data=json.dumps(data),
@@ -503,6 +630,86 @@ class SCAClient(Client):
         if not response.ok:
             raise errors.StoreSnapBuildError(response)
 
+    def snap_revisions(self, snap_id, series, arch):
+        qs = {}
+        if series:
+            qs['series'] = series
+        if arch:
+            qs['arch'] = arch
+        url = 'snaps/' + snap_id + '/history'
+        if qs:
+            url += '?' + urllib.parse.urlencode(qs)
+        auth = _macaroon_auth(self.conf)
+        response = self.get(
+            url,
+            headers={'Authorization': auth,
+                     'Content-Type': 'application/json',
+                     'Accept': 'application/json'})
+        if not response.ok:
+            raise errors.StoreSnapRevisionsError(
+                response, snap_id, series, arch)
+
+        response_json = response.json()
+
+        return response_json
+
+    def snap_status(self, snap_id, series, arch):
+        qs = {}
+        if series:
+            qs['series'] = series
+        if arch:
+            qs['architecture'] = arch
+        url = 'snaps/' + snap_id + '/state'
+        if qs:
+            url += '?' + urllib.parse.urlencode(qs)
+        auth = _macaroon_auth(self.conf)
+        response = self.get(
+            url,
+            headers={'Authorization': auth,
+                     'Content-Type': 'application/json',
+                     'Accept': 'application/json'})
+        if not response.ok:
+            raise errors.StoreSnapStatusError(response, snap_id, series, arch)
+
+        response_json = response.json()
+
+        return response_json
+
+    def close_channels(self, snap_id, channel_names):
+        url = 'snaps/{}/close'.format(snap_id)
+        data = {
+            'channels': channel_names
+        }
+        headers = {
+            'Authorization': _macaroon_auth(self.conf),
+        }
+        response = self.post(url, json=data, headers=headers)
+        if not response.ok:
+            raise errors.StoreChannelClosingError(response)
+
+        try:
+            results = response.json()
+            return results['closed_channels'], results['channel_map_tree']
+        except (JSONDecodeError, KeyError):
+            logger.debug(
+                'Invalid response from the server on channel closing:\n'
+                '{} {}\n{}'.format(response.status_code, response.reason,
+                                   response.content))
+            raise errors.StoreChannelClosingError(response)
+
+    def sign_developer_agreement(self, latest_tos_accepted=False):
+        auth = _macaroon_auth(self.conf)
+        data = {'latest_tos_accepted': latest_tos_accepted}
+        response = self.post(
+            'agreement/', data=json.dumps(data),
+            headers={'Authorization': auth,
+                     'Content-Type': 'application/json',
+                     'Accept': 'application/json'})
+
+        if not response.ok:
+            raise errors.DeveloperAgreementSignError(response)
+        return response.json()
+
 
 class StatusTracker:
 
@@ -510,11 +717,13 @@ class StatusTracker:
         'being_processed': 'Processing...',
         'ready_to_release': 'Ready to release!',
         'need_manual_review': 'Will need manual review...',
+        'processing_upload_delta_error': 'Error while processing delta...',
         'processing_error': 'Error while processing...',
     }
 
     __error_codes = (
         'processing_error',
+        'processing_upload_delta_error',
         'need_manual_review',
     )
 
