@@ -38,6 +38,17 @@ class BaseHTTPRequestHandler(http.server.BaseHTTPRequestHandler):
         logger.debug(args)
 
 
+class FakeFileHTTPRequestHandler(BaseHTTPRequestHandler):
+
+    def do_GET(self):
+        data = 'Test fake compressed file'
+        self.send_response(200)
+        self.send_header('Content-Length', len(data))
+        self.send_header('Content-type', 'text/html')
+        self.end_headers()
+        self.wfile.write(data.encode())
+
+
 class FakePartsServer(http.server.HTTPServer):
 
     def __init__(self, server_address):
@@ -322,41 +333,52 @@ class FakeStoreAPIRequestHandler(BaseHTTPRequestHandler):
 
     _DEV_API_PATH = '/dev/api/'
 
-    def do_POST(self):
-        if self.server.fake_store.needs_refresh:
-            self._handle_needs_refresh()
-            return
+    def do_POST(self):  # noqa: C901
+        self._handle_refresh()
         parsed_path = urllib.parse.urlparse(self.path)
         acl_path = urllib.parse.urljoin(self._DEV_API_PATH, 'acl/')
         account_key_path = urllib.parse.urljoin(
             self._DEV_API_PATH, 'account/account-key')
-        sign_build_path = urllib.parse.urljoin(
-            self._DEV_API_PATH, 'snaps/snap-id/builds'.format())
+        snap_id_path = urllib.parse.urljoin(
+            self._DEV_API_PATH, 'snaps/')
         register_path = urllib.parse.urljoin(
             self._DEV_API_PATH, 'register-name/')
-        upload_path = urllib.parse.urljoin(self._DEV_API_PATH, 'snap-push/')
-        release_path = urllib.parse.urljoin(self._DEV_API_PATH,
-                                            'snap-release/')
+        upload_path = urllib.parse.urljoin(
+            self._DEV_API_PATH, 'snap-push/')
+        release_path = urllib.parse.urljoin(
+            self._DEV_API_PATH, 'snap-release/')
+        agreement_path = urllib.parse.urljoin(
+            self._DEV_API_PATH, 'agreement/')
 
         if parsed_path.path.startswith(acl_path):
             permission = parsed_path.path[len(acl_path):].strip('/')
             self._handle_acl_request(permission)
         elif parsed_path.path == account_key_path:
             self._handle_account_key_request()
-        elif parsed_path.path == sign_build_path:
-            self._handle_sign_build_request()
+        elif parsed_path.path.startswith(snap_id_path):
+            if parsed_path.path.endswith('/builds'):
+                self._handle_sign_build_request()
+            elif parsed_path.path.endswith('/close'):
+                self._handle_close_request()
         elif parsed_path.path.startswith(register_path):
             self._handle_registration_request()
         elif parsed_path.path.startswith(upload_path):
             self._handle_upload_request()
         elif parsed_path.path.startswith(release_path):
             self._handle_release_request()
+        elif parsed_path.path.startswith(agreement_path):
+            self._handle_sign_request()
         else:
             logger.error(
                 'Not implemented path in fake Store API server: {}'.format(
                     self.path))
             raise NotImplementedError(self.path)
 
+    def _handle_refresh(self):
+        if self.server.fake_store.needs_refresh:
+            self._handle_needs_refresh()
+            return
+
     def _handle_needs_refresh(self):
         self.send_response(401)
         self.send_header('WWW-Authenticate', 'Macaroon needs_refresh=1')
@@ -374,18 +396,20 @@ class FakeStoreAPIRequestHandler(BaseHTTPRequestHandler):
         self.send_response(200)
         self.send_header('Content-Type', 'application/json')
         self.end_headers()
+        sso_host = urllib.parse.urlparse(os.environ.get(
+            'UBUNTU_SSO_API_ROOT_URL', "http://localhost")).netloc
         macaroon = pymacaroons.Macaroon(
             caveats=[
                 pymacaroons.Caveat(
                     caveat_id='test caveat',
-                    location='localhost',
+                    location=sso_host,
                     verification_key_id='test verifiacion')
             ])
         response = {'macaroon': macaroon.serialize()}
         self.wfile.write(json.dumps(response).encode())
 
     def _handle_sign_build_request(self):
-        logger.debug('Handling snap-build request')
+        logger.debug('Handling sign-build request')
         string_data = self.rfile.read(
             int(self.headers['Content-Length'])).decode('utf8')
         snap_build = json.loads(string_data)['assertion']
@@ -426,6 +450,64 @@ class FakeStoreAPIRequestHandler(BaseHTTPRequestHandler):
         self.end_headers()
         self.wfile.write(content)
 
+    def _handle_close_request(self):
+        logger.debug('Handling close request')
+        string_data = self.rfile.read(
+            int(self.headers['Content-Length'])).decode('utf8')
+        payload = json.loads(string_data)
+        channels = payload['channels']
+
+        if channels == ['invalid']:
+            self.send_response(400)
+            self.send_header('Content-Type', 'application/json')
+            error = {
+                'error_list': [
+                    {'code': 'invalid-field',
+                     'message': ('The \'channels\' field content is not '
+                                 'valid.')},
+                ],
+            }
+            content = json.dumps(error).encode()
+        elif channels == ['unexpected']:
+            self.send_response(500)
+            self.send_header('Content-Type', 'text/plain')
+            content = b'unexpected chunk of data'
+        elif channels == ['broken-plain']:
+            self.send_response(200)
+            self.send_header('Content-Type', 'text/plain')
+            content = b'plain data'
+        elif channels == ['broken-json']:
+            self.send_response(200)
+            self.send_header('Content-Type', 'application/json')
+            response = {
+                'closed_channels': channels,
+            }
+            content = json.dumps(response).encode()
+        else:
+            self.send_response(200)
+            self.send_header('Content-Type', 'application/json')
+            response = {
+                'closed_channels': channels,
+                'channel_map_tree': {
+                    'latest': {
+                        '16': {
+                            'amd64': [
+                                {'channel': 'stable', 'info': 'none'},
+                                {'channel': 'candidate', 'info': 'none'},
+                                {'channel': 'beta', 'info': 'specific',
+                                 'version': '1.1', 'revision': 42},
+                                {'channel': 'edge', 'info': 'tracking'}
+                            ]
+
+                        }
+                    }
+                },
+            }
+            content = json.dumps(response).encode()
+
+        self.end_headers()
+        self.wfile.write(content)
+
     def _handle_account_key_request(self):
         string_data = self.rfile.read(
             int(self.headers['Content-Length'])).decode('utf8')
@@ -486,20 +568,23 @@ class FakeStoreAPIRequestHandler(BaseHTTPRequestHandler):
         logger.debug(
             'Handling registration request with content {}'.format(data))
         snap_name = data['snap_name']
+        is_private = data['is_private']
 
         if data['snap_name'] == 'test-already-registered-snap-name':
             self._handle_register_409('already_registered')
         elif data['snap_name'] == 'test-reserved-snap-name':
             self._handle_register_409('reserved_name')
+        elif data['snap_name'] == 'test-already-owned-snap-name':
+            self._handle_register_409('already_owned')
         elif data['snap_name'].startswith('test-too-fast'):
             self._handle_register_429('register_window')
         elif data['snap_name'] == 'snap-name-no-clear-error':
             self._handle_unclear_registration_error()
         else:
-            self._handle_successful_registration(snap_name)
+            self._handle_successful_registration(snap_name, is_private)
 
-    def _handle_successful_registration(self, name):
-        self.server.registered_names.append(name)
+    def _handle_successful_registration(self, name, is_private):
+        self.server.registered_names.append((name, is_private))
         self.send_response(201)
         self.send_header('Content-Type', 'application/json')
         self.end_headers()
@@ -552,6 +637,8 @@ class FakeStoreAPIRequestHandler(BaseHTTPRequestHandler):
             details_path = 'details/upload-id/review-snap'
         elif data['name'] == 'test-snap-unregistered':
             response_code = 404
+        elif data['name'] == 'test-duplicate-snap':
+            details_path = 'details/upload-id/duplicate-snap'
 
         logger.debug('Handling upload request')
         self.send_response(response_code)
@@ -601,6 +688,55 @@ class FakeStoreAPIRequestHandler(BaseHTTPRequestHandler):
                 ]
             }
             data = json.dumps(response).encode()
+        elif data['name'].startswith('arm-'):
+            response = {
+                'opened_channels': data['channels'],
+                'channel_map_tree': {
+                    '0.1': {
+                        '16': {
+                            'armhf':
+                                [
+                                    {'channel': 'stable', 'info': 'none'},
+                                    {'channel': 'candidate', 'info': 'none'},
+                                    {'revision': int(data['revision']),
+                                     'channel': 'beta',
+                                     'version': '0', 'info': 'specific'},
+                                    {'channel': 'edge', 'info': 'tracking'}
+                                ]
+                        }
+                    }
+                }
+            }
+            data = json.dumps(response).encode()
+        elif data['name'].startswith('multiarch-'):
+            response = {
+                'opened_channels': data['channels'],
+                'channel_map_tree': {
+                    '0.1': {
+                        '16': {
+                            'amd64':
+                                [
+                                    {'channel': 'stable', 'info': 'none'},
+                                    {'channel': 'candidate', 'info': 'none'},
+                                    {'revision': int(data['revision']),
+                                     'channel': 'beta',
+                                     'version': '0', 'info': 'specific'},
+                                    {'channel': 'edge', 'info': 'tracking'}
+                                ],
+                            'armhf':
+                                [
+                                    {'channel': 'stable', 'info': 'none'},
+                                    {'channel': 'candidate', 'info': 'none'},
+                                    {'revision': int(data['revision']),
+                                     'channel': 'beta',
+                                     'version': '0', 'info': 'specific'},
+                                    {'channel': 'edge', 'info': 'tracking'}
+                                ]
+                        }
+                    }
+                }
+            }
+            data = json.dumps(response).encode()
         else:
             raise NotImplementedError(
                 'Cannot handle release request for {!r}'.format(data['name']))
@@ -612,7 +748,45 @@ class FakeStoreAPIRequestHandler(BaseHTTPRequestHandler):
 
         self.wfile.write(data)
 
-    def do_GET(self):
+    def _handle_sign_request(self):
+        string_data = self.rfile.read(
+            int(self.headers['Content-Length'])).decode('utf8')
+        data = json.loads(string_data)
+
+        if 'STORE_DOWN' in os.environ:
+            response_code = 500
+            content_type = 'text/plain'
+            response = b'Broken'
+        else:
+            if data['latest_tos_accepted'] is not True:
+                response_code = 400
+                content_type = 'application/json'
+                content = {
+                    "error_list": [{
+                        "message": "`latest_tos_accepted` must be `true`",
+                        "code": "bad-request",
+                        "extra": {"latest_tos_accepted": "true"}}]}
+                response = json.dumps(content).encode()
+            else:
+                response_code = 200
+                content_type = 'application/json'
+                content = {"content": {
+                    "latest_tos_accepted": True,
+                    "tos_url": 'http://fake-url.com',
+                    "latest_tos_date": '2000-01-01',
+                    "accepted_tos_date": '2010-10-10'
+                    }
+                }
+                response = json.dumps(content).encode()
+
+        self.send_response(response_code)
+        self.send_header('Content-Type', content_type)
+        self.end_headers()
+        self.wfile.write(response)
+
+    # This function's complexity is correlated to the number of
+    # url paths, no point in checking that.
+    def do_GET(self):  # noqa: C901
         if self.server.fake_store.needs_refresh:
             self._handle_needs_refresh()
             return
@@ -621,9 +795,14 @@ class FakeStoreAPIRequestHandler(BaseHTTPRequestHandler):
             self._DEV_API_PATH, '/details/upload-id/good-snap')
         details_review = urllib.parse.urljoin(
             self._DEV_API_PATH, '/details/upload-id/review-snap')
+        duplicate_error = urllib.parse.urljoin(
+            self._DEV_API_PATH, '/details/upload-id/duplicate-snap')
         account_path = urllib.parse.urljoin(self._DEV_API_PATH, 'account')
+        snap_path = urllib.parse.urljoin(self._DEV_API_PATH, 'snaps')
         good_validations_path = urllib.parse.urljoin(
             self._DEV_API_PATH, 'snaps/good/validations')
+        no_validations_path = urllib.parse.urljoin(
+            self._DEV_API_PATH, 'snaps/snap-id/validations')
         bad_validations_path = urllib.parse.urljoin(
             self._DEV_API_PATH, 'snaps/bad/validations')
         err_validations_path = urllib.parse.urljoin(
@@ -633,6 +812,8 @@ class FakeStoreAPIRequestHandler(BaseHTTPRequestHandler):
             self._handle_scan_complete_request('ready_to_release', True)
         elif parsed_path.path.startswith(details_review):
             self._handle_scan_complete_request('need_manual_review', False)
+        elif parsed_path.path.startswith(duplicate_error):
+            self._handle_duplicate_snap_request('processing_error', False)
         elif parsed_path.path == account_path:
             self._handle_account_request()
         elif parsed_path.path.startswith(good_validations_path):
@@ -641,6 +822,13 @@ class FakeStoreAPIRequestHandler(BaseHTTPRequestHandler):
             self._handle_validation_request('bad')
         elif parsed_path.path.startswith(err_validations_path):
             self._handle_validation_request('err')
+        elif parsed_path.path.startswith(no_validations_path):
+            self._handle_validation_request('no')
+        elif parsed_path.path.startswith(snap_path):
+            if parsed_path.path.endswith('/history'):
+                self._handle_snap_revisions()
+            elif parsed_path.path.endswith('/state'):
+                self._handle_snap_status()
         else:
             logger.error(
                 'Not implemented path in fake Store API server: {}'.format(
@@ -660,7 +848,8 @@ class FakeStoreAPIRequestHandler(BaseHTTPRequestHandler):
                 "snap-id": "snap-id-gating",
                 "timestamp": "2016-09-19T21:07:27.756001Z",
                 "type": "validation",
-                "revoked": "false"
+                "revoked": "false",
+                "required": True,
             }, {
                 "approved-snap-id": "snap-id-2",
                 "approved-snap-revision": "5",
@@ -671,13 +860,29 @@ class FakeStoreAPIRequestHandler(BaseHTTPRequestHandler):
                 "snap-id": "snap-id-gating",
                 "timestamp": "2016-09-19T21:07:27.756001Z",
                 "type": "validation",
-                "revoked": "false"
+                "revoked": "false",
+                "required": False,
+            }, {
+                "approved-snap-id": "snap-id-3",
+                "approved-snap-revision": "-",
+                "approved-snap-name": "snap-3",
+                "authority-id": "dev-1",
+                "series": "16",
+                "sign-key-sha3-384": "1234567890",
+                "snap-id": "snap-id-gating",
+                "timestamp": "2016-09-19T21:07:27.756001Z",
+                "type": "validation",
+                "revoked": "false",
+                "required": True,
             }]
             response = json.dumps(response).encode()
             status = 200
         elif code == 'bad':
             response = 'foo'.encode()
             status = 200
+        elif code == 'no':
+            response = json.dumps([]).encode()
+            status = 200
         elif code == 'err':
             status = 503
             response = {'error_list': [{'message': 'error'}]}
@@ -687,6 +892,23 @@ class FakeStoreAPIRequestHandler(BaseHTTPRequestHandler):
         self.end_headers()
         self.wfile.write(response)
 
+    def _handle_duplicate_snap_request(self, code, can_release):
+        logger.debug('Handling duplicate snap request')
+        self.send_response(200)
+        self.send_header('Content-Type', 'application/json')
+        self.end_headers()
+        response = {
+            'code': code,
+            'url': '/dev/click-apps/5349/rev/1',
+            'can_release': can_release,
+            'revision': '1',
+            'processed': True,
+            'errors': [
+                {'message': 'Duplicate snap already uploaded'},
+            ]
+        }
+        self.wfile.write(json.dumps(response).encode())
+
     def _handle_scan_complete_request(self, code, can_release):
         logger.debug('Handling scan complete request')
         self.send_response(200)
@@ -707,18 +929,126 @@ class FakeStoreAPIRequestHandler(BaseHTTPRequestHandler):
         self.send_header('Content-Type', 'application/json')
         self.end_headers()
         snaps = {
-            'basic': {'snap-id': 'snap-id'},
-            'ubuntu-core': {'snap-id': 'good'},
+            'basic': {'snap-id': 'snap-id', 'status': 'Approved',
+                      'private': False, 'price': None,
+                      'since': '2016-12-12T01:01:01Z'},
+            'ubuntu-core': {'snap-id': 'good', 'status': 'Approved',
+                            'private': False, 'price': None,
+                            'since': '2016-12-12T01:01:01Z'},
             }
         snaps.update({
-            n: {'snap-id': 'fake-snap-id'}
-            for n in self.server.registered_names})
+            name: {'snap-id': 'fake-snap-id', 'status': 'Approved',
+                   'private': private, 'price': None,
+                   'since': '2016-12-12T01:01:01Z'}
+            for name, private in self.server.registered_names})
         self.wfile.write(json.dumps({
             'account_id': 'abcd',
             'account_keys': self.server.account_keys,
             'snaps': {'16': snaps},
         }).encode())
 
+    def _handle_snap_revisions(self):
+        logger.debug('Handling account request')
+        self.send_response(200)
+        self.send_header('Content-Type', 'application/json')
+        self.end_headers()
+        revisions = [{
+            'series': ['16'],
+            'channels': [],
+            'version': '2.0.1',
+            'timestamp': '2016-09-27T19:23:40Z',
+            'current_channels': ['beta', 'edge'],
+            'arch': 'i386',
+            'revision': 2
+        }, {
+            'series': ['16'],
+            'channels': ['stable', 'edge'],
+            'version': '2.0.2',
+            'timestamp': '2016-09-27T18:38:43Z',
+            'current_channels': ['stable', 'candidate', 'beta'],
+            'arch': 'amd64',
+            'revision': 1,
+        }]
+
+        parsed_qs = urllib.parse.parse_qs(
+            urllib.parse.urlparse(self.path).query)
+        if 'arch' in parsed_qs:
+            output = [
+                rev for rev in revisions if rev['arch'] in parsed_qs['arch']]
+        else:
+            output = revisions
+        self.wfile.write(json.dumps(output).encode())
+
+    def _handle_snap_status(self):
+        logger.debug('Handling account request')
+        self.send_response(200)
+        self.send_header('Content-Type', 'application/json')
+        self.end_headers()
+        channel_map = {
+            'channel_map_tree': {
+                'latest': {
+                    '16': {
+                        'i386': [
+                            {
+                                'info': 'none',
+                                'channel': 'stable'
+                            },
+                            {
+                                'info': 'none',
+                                'channel': 'beta'
+                            },
+                            {
+                                'info': 'specific',
+                                'version': '1.0-i386',
+                                'channel': 'edge',
+                                'revision': 3
+                            },
+                        ],
+                        'amd64': [
+                            {
+                                'info': 'specific',
+                                'version': '1.0-amd64',
+                                'channel': 'stable',
+                                'revision': 2
+                            },
+                            {
+                                'info': 'specific',
+                                'version': '1.1-amd64',
+                                'channel': 'beta',
+                                'revision': 4
+                            },
+                            {
+                                'info': 'tracking',
+                                'channel': 'edge'
+                            },
+                        ],
+
+                    }
+                }
+            }
+        }
+
+        parsed_qs = urllib.parse.parse_qs(
+            urllib.parse.urlparse(self.path).query)
+        if 'architecture' in parsed_qs:
+            arch = parsed_qs['architecture'][0]
+            series = channel_map['channel_map_tree']['latest']['16']
+            if arch in series:
+                output = {
+                    'channel_map_tree': {
+                        'latest': {
+                            '16': {
+                                arch: series[arch]
+                            }
+                        }
+                    }
+                }
+            else:
+                output = {}
+        else:
+            output = channel_map
+        self.wfile.write(json.dumps(output).encode())
+
     def do_PUT(self):
         if self.server.fake_store.needs_refresh:
             self._handle_needs_refresh()
@@ -796,6 +1126,10 @@ class FakeStoreSearchRequestHandler(BaseHTTPRequestHandler):
         logger.debug(
             'Handling details request for package {}, with headers {}'.format(
                 package, self.headers))
+        if 'User-Agent' not in self.headers:
+            self.send_response(500)
+            self.end_headers()
+            return
         response = self._get_details_response(package)
         if response is None:
             self.send_response(404)
@@ -808,20 +1142,31 @@ class FakeStoreSearchRequestHandler(BaseHTTPRequestHandler):
 
     def _get_details_response(self, package):
         # ubuntu-core is used in integration tests with fake servers.
+
+        # sha512sum snapcraft/tests/data/test-snap.snap
+        test_sha512 = (
+            '69d57dcacf4f126592d4e6ff689ad8bb8a083c7b9fe44f6e738ef'
+            'd22a956457f14146f7f067b47bd976cf0292f2993ad864ccb498b'
+            'fda4128234e4c201f28fe9')
+
         if package in ('test-snap', 'ubuntu-core'):
-            # sha512sum snapcraft/tests/data/test-snap.snap
-            sha512 = (
-                '69d57dcacf4f126592d4e6ff689ad8bb8a083c7b9fe44f6e738ef'
-                'd22a956457f14146f7f067b47bd976cf0292f2993ad864ccb498b'
-                'fda4128234e4c201f28fe9')
+            sha512 = test_sha512
         elif package == 'test-snap-with-wrong-sha':
             sha512 = 'wrong sha'
+        elif package == 'test-snap-branded-store':
+            # Branded-store snaps require Store pinning and authorization.
+            if (self.headers.get('X-Ubuntu-Store') != 'Test-Branded' or
+                    self.headers.get('Authorization') is None):
+                return None
+            sha512 = test_sha512
         else:
             return None
+
         response = {
-            'download_url': urllib.parse.urljoin(
+            'anon_download_url': urllib.parse.urljoin(
                 'http://localhost:{}'.format(self.server.server_port),
                 'download-snap/test-snap.snap'),
+            'download_sha3_384': '1234567890',
             'download_sha512': sha512,
             'snap_id': 'good',
             'developer_id': package + '-developer-id',
@@ -831,6 +1176,10 @@ class FakeStoreSearchRequestHandler(BaseHTTPRequestHandler):
 
     def _handle_download_request(self, snap):
         logger.debug('Handling download request for snap {}'.format(snap))
+        if 'User-Agent' not in self.headers:
+            self.send_repsonse(500)
+            self.end_headers()
+            return
         self.send_response(200)
         self.send_header('Content-Type', 'application/octet-stream')
         self.end_headers()
