@@ -1,6 +1,6 @@
 # -*- Mode:Python; indent-tabs-mode:nil; tab-width:4 -*-
 #
-# Copyright (C) 2015 Canonical Ltd
+# Copyright (C) 2015-2017 Canonical Ltd
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License version 3 as
@@ -14,16 +14,26 @@
 # You should have received a copy of the GNU General Public License
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
+import contextlib
+import fileinput
 import logging
-
-import fixtures
+import os
+import re
+import shutil
 from unittest import mock
 
-import snapcraft
-from snapcraft.internal import (
-    pluginhandler,
-    lifecycle,
+import fixtures
+from testtools.matchers import (
+    FileContains,
+    FileExists,
+    MatchesRegex,
+    Not,
 )
+
+import snapcraft
+from snapcraft import storeapi
+from snapcraft.file_utils import calculate_sha3_384
+from snapcraft.internal import pluginhandler, lifecycle
 from snapcraft import tests
 
 
@@ -79,12 +89,14 @@ grade: stable
       - part1
 """)
 
-        with self.assertRaises(RuntimeError) as raised:
-            lifecycle.execute('pull', self.project_options,
-                              part_names=['part2'])
+        raised = self.assertRaises(
+            RuntimeError,
+            lifecycle.execute,
+            'pull', self.project_options,
+            part_names=['part2'])
 
         self.assertEqual(
-            raised.exception.__str__(),
+            raised.__str__(),
             "Requested 'pull' of 'part2' but there are unsatisfied "
             "prerequisites: 'part1'")
 
@@ -192,13 +204,15 @@ grade: stable
         self.fake_logger = fixtures.FakeLogger(level=logging.INFO)
         self.useFixture(self.fake_logger)
 
-        def _fake_is_dirty(self, step):
-            return self.name == 'part1' and step == 'prime'
+        def _fake_dirty_report(self, step):
+            if self.name == 'part1' and step == 'prime':
+                return pluginhandler.DirtyReport({'foo'}, {'bar'})
+            return None
 
         # Should automatically clean and re-prime if that step is dirty
         # for the part.
-        with mock.patch.object(pluginhandler.PluginHandler, 'is_dirty',
-                               _fake_is_dirty):
+        with mock.patch.object(pluginhandler.PluginHandler, 'get_dirty_report',
+                               _fake_dirty_report):
             lifecycle.execute('prime', self.project_options)
 
         output = self.fake_logger.output.split('\n')
@@ -239,13 +253,15 @@ grade: stable
         self.fake_logger = fixtures.FakeLogger(level=logging.INFO)
         self.useFixture(self.fake_logger)
 
-        def _fake_is_dirty(self, step):
-            return step == 'prime'
+        def _fake_dirty_report(self, step):
+            if step == 'prime':
+                return pluginhandler.DirtyReport({'foo'}, {'bar'})
+            return None
 
         # Should automatically clean and re-prime if that step is dirty
         # for the part.
-        with mock.patch.object(pluginhandler.PluginHandler, 'is_dirty',
-                               _fake_is_dirty):
+        with mock.patch.object(pluginhandler.PluginHandler, 'get_dirty_report',
+                               _fake_dirty_report):
             lifecycle.execute('prime', self.project_options)
 
         output = self.fake_logger.output.split('\n')
@@ -287,13 +303,15 @@ grade: stable
         self.fake_logger = fixtures.FakeLogger(level=logging.INFO)
         self.useFixture(self.fake_logger)
 
-        def _fake_is_dirty(self, step):
-            return self.name == 'part1' and step == 'stage'
+        def _fake_dirty_report(self, step):
+            if self.name == 'part1' and step == 'stage':
+                return pluginhandler.DirtyReport({'foo'}, {'bar'})
+            return None
 
         # Should automatically clean and re-stage if that step is dirty
         # for the part.
-        with mock.patch.object(pluginhandler.PluginHandler, 'is_dirty',
-                               _fake_is_dirty):
+        with mock.patch.object(pluginhandler.PluginHandler, 'get_dirty_report',
+                               _fake_dirty_report):
             lifecycle.execute('stage', self.project_options)
 
         output = self.fake_logger.output.split('\n')
@@ -334,13 +352,15 @@ grade: stable
         self.fake_logger = fixtures.FakeLogger(level=logging.INFO)
         self.useFixture(self.fake_logger)
 
-        def _fake_is_dirty(self, step):
-            return step == 'stage'
+        def _fake_dirty_report(self, step):
+            if step == 'stage':
+                return pluginhandler.DirtyReport({'foo'}, {'bar'})
+            return None
 
         # Should automatically clean and re-stage if that step is dirty
         # for the part.
-        with mock.patch.object(pluginhandler.PluginHandler, 'is_dirty',
-                               _fake_is_dirty):
+        with mock.patch.object(pluginhandler.PluginHandler, 'get_dirty_report',
+                               _fake_dirty_report):
             lifecycle.execute('stage', self.project_options)
 
         output = self.fake_logger.output.split('\n')
@@ -387,16 +407,20 @@ grade: stable
         self.fake_logger = fixtures.FakeLogger(level=logging.INFO)
         self.useFixture(self.fake_logger)
 
-        def _fake_is_dirty(self, step):
-            return step == 'stage'
+        def _fake_dirty_report(self, step):
+            if step == 'stage':
+                return pluginhandler.DirtyReport({'foo'}, {'bar'})
+            return None
 
         # Should raise a RuntimeError about the fact that stage is dirty but
         # it has dependents that need it.
-        with mock.patch.object(pluginhandler.PluginHandler, 'is_dirty',
-                               _fake_is_dirty):
-            with self.assertRaises(RuntimeError) as raised:
-                lifecycle.execute('stage', self.project_options,
-                                  part_names=['part1'])
+        with mock.patch.object(pluginhandler.PluginHandler, 'get_dirty_report',
+                               _fake_dirty_report):
+            raised = self.assertRaises(
+                RuntimeError,
+                lifecycle.execute,
+                'stage', self.project_options,
+                part_names=['part1'])
 
         output = self.fake_logger.output.split('\n')
         part1_output = [line.strip() for line in output if 'part1' in line]
@@ -410,7 +434,7 @@ grade: stable
         self.assertEqual(
             "The 'stage' step for 'part1' needs to be run again, but 'part2' "
             "depends upon it. Please clean the build step of 'part2' first.",
-            str(raised.exception))
+            str(raised))
 
     def test_dirty_stage_part_with_unbuilt_dependent(self):
         self.make_snapcraft_yaml("""parts:
@@ -428,13 +452,15 @@ grade: stable
         self.fake_logger = fixtures.FakeLogger(level=logging.INFO)
         self.useFixture(self.fake_logger)
 
-        def _fake_is_dirty(self, step):
-            return step == 'stage'
+        def _fake_dirty_report(self, step):
+            if step == 'stage':
+                return pluginhandler.DirtyReport({'foo'}, {'bar'})
+            return None
 
         # Should automatically clean and re-stage if that step is dirty
         # for the part.
-        with mock.patch.object(pluginhandler.PluginHandler, 'is_dirty',
-                               _fake_is_dirty):
+        with mock.patch.object(pluginhandler.PluginHandler, 'get_dirty_report',
+                               _fake_dirty_report):
             lifecycle.execute('stage', self.project_options,
                               part_names=['part1'])
 
@@ -464,13 +490,15 @@ grade: stable
         self.fake_logger = fixtures.FakeLogger(level=logging.INFO)
         self.useFixture(self.fake_logger)
 
-        def _fake_is_dirty(self, step):
-            return step == 'stage'
+        def _fake_dirty_report(self, step):
+            if step == 'stage':
+                return pluginhandler.DirtyReport({'foo'}, {'bar'})
+            return None
 
         # Should automatically clean and re-stage if that step is dirty
         # for the part.
-        with mock.patch.object(pluginhandler.PluginHandler, 'is_dirty',
-                               _fake_is_dirty):
+        with mock.patch.object(pluginhandler.PluginHandler, 'get_dirty_report',
+                               _fake_dirty_report):
             lifecycle.execute('prime', self.project_options)
 
         self.assertEqual(
@@ -495,22 +523,29 @@ grade: stable
         self.fake_logger = fixtures.FakeLogger(level=logging.INFO)
         self.useFixture(self.fake_logger)
 
-        def _fake_is_dirty(self, step):
-            return step == 'build'
+        def _fake_dirty_report(self, step):
+            if step == 'build':
+                return pluginhandler.DirtyReport({'foo', 'bar'}, set())
+            return None
 
         # Should catch that the part needs to be rebuilt and raise an error.
-        with mock.patch.object(pluginhandler.PluginHandler, 'is_dirty',
-                               _fake_is_dirty):
-            with self.assertRaises(RuntimeError) as raised:
-                lifecycle.execute('build', self.project_options)
+        with mock.patch.object(pluginhandler.PluginHandler, 'get_dirty_report',
+                               _fake_dirty_report):
+            raised = self.assertRaises(
+                RuntimeError,
+                lifecycle.execute,
+                'build', self.project_options)
 
         self.assertEqual(
             'Skipping pull part1 (already ran)\n',
             self.fake_logger.output)
 
         self.assertEqual(
-            "The 'build' step of 'part1' is out of date. Please clean that "
-            "part's 'build' step in order to rebuild", str(raised.exception))
+            "The 'build' step of 'part1' is out of date:\n\n"
+            "The 'bar' and 'foo' part properties appear to have changed.\n\n"
+            "In order to continue, please clean that part's 'build' step "
+            "by running: snapcraft clean part1 -s build\n",
+            str(raised))
 
     def test_dirty_pull_raises(self):
         self.make_snapcraft_yaml("""parts:
@@ -525,23 +560,30 @@ grade: stable
         self.fake_logger = fixtures.FakeLogger(level=logging.INFO)
         self.useFixture(self.fake_logger)
 
-        def _fake_is_dirty(self, step):
-            return step == 'pull'
+        def _fake_dirty_report(self, step):
+            if step == 'pull':
+                return pluginhandler.DirtyReport(set(), {'foo', 'bar'})
+            return None
 
         # Should catch that the part needs to be re-pulled and raise an error.
-        with mock.patch.object(pluginhandler.PluginHandler, 'is_dirty',
-                               _fake_is_dirty):
-            with self.assertRaises(RuntimeError) as raised:
-                lifecycle.execute('pull', self.project_options)
+        with mock.patch.object(pluginhandler.PluginHandler, 'get_dirty_report',
+                               _fake_dirty_report):
+            raised = self.assertRaises(
+                RuntimeError,
+                lifecycle.execute,
+                'pull', self.project_options)
 
         self.assertEqual('', self.fake_logger.output)
 
         self.assertEqual(
-            "The 'pull' step of 'part1' is out of date. Please clean that "
-            "part's 'pull' step in order to rebuild", str(raised.exception))
+            "The 'pull' step of 'part1' is out of date:\n\n"
+            "The 'bar' and 'foo' project options appear to have changed.\n\n"
+            "In order to continue, please clean that part's 'pull' step "
+            "by running: snapcraft clean part1 -s pull\n",
+            str(raised))
 
     @mock.patch.object(snapcraft.BasePlugin, 'enable_cross_compilation')
-    @mock.patch('snapcraft.repo.install_build_packages')
+    @mock.patch('snapcraft.repo.Repo.install_build_packages')
     def test_pull_is_dirty_if_target_arch_changes(
             self, mock_install_build_packages, mock_enable_cross_compilation):
         self.make_snapcraft_yaml("""parts:
@@ -560,42 +602,131 @@ grade: stable
         # Pull it again with armhf. Should catch that the part needs to be
         # re-pulled due to the change in target architecture and raise an
         # error.
-        with self.assertRaises(RuntimeError) as raised:
-            lifecycle.execute('pull', snapcraft.ProjectOptions(
+        raised = self.assertRaises(
+            RuntimeError,
+            lifecycle.execute,
+            'pull', snapcraft.ProjectOptions(
                 target_deb_arch='armhf'))
 
         self.assertEqual("Setting target machine to 'armhf'\n",
                          self.fake_logger.output)
 
         self.assertEqual(
-            "The 'pull' step of 'part1' is out of date. Please clean that "
-            "part's 'pull' step in order to rebuild", str(raised.exception))
+            "The 'pull' step of 'part1' is out of date:\n\n"
+            "The 'deb_arch' project option appears to have changed.\n\n"
+            "In order to continue, please clean that part's 'pull' step "
+            "by running: snapcraft clean part1 -s pull\n",
+            str(raised))
 
 
-class HumanizeListTestCases(tests.TestCase):
+class CoreSetupTestCase(tests.TestCase):
 
-    def test_no_items(self):
-        items = []
-        output = lifecycle._humanize_list(items)
-        self.assertEqual(output, '')
+    def setUp(self):
+        super().setUp()
 
-    def test_one_item(self):
-        items = ['foo']
-        output = lifecycle._humanize_list(items)
-        self.assertEqual(output, "'foo'")
+        self.core_path = os.path.join(self.path, 'core', 'current')
+        patcher = mock.patch('snapcraft.internal.common.get_core_path')
+        core_path_mock = patcher.start()
+        core_path_mock.return_value = self.core_path
+        self.addCleanup(patcher.stop)
 
-    def test_two_items(self):
-        items = ['foo', 'bar']
-        output = lifecycle._humanize_list(items)
-        self.assertEqual(output, "'bar' and 'foo'",
-                         "Expected 'bar' before 'foo' due to sorting")
+        patcher = mock.patch.object(snapcraft.ProjectOptions,
+                                    'get_core_dynamic_linker')
+        get_linker_mock = patcher.start()
+        get_linker_mock.return_value = '/lib/ld'
+        self.addCleanup(patcher.stop)
 
-    def test_three_items(self):
-        items = ['foo', 'bar', 'baz']
-        output = lifecycle._humanize_list(items)
-        self.assertEqual(output, "'bar', 'baz', and 'foo'")
+        self.tempdir = os.path.join(self.path, 'tmpdir')
+        patcher = mock.patch('snapcraft.internal.lifecycle.TemporaryDirectory')
+        self.tempdir_mock = patcher.start()
+        self.addCleanup(patcher.stop)
 
-    def test_four_items(self):
-        items = ['foo', 'bar', 'baz', 'qux']
-        output = lifecycle._humanize_list(items)
-        self.assertEqual(output, "'bar', 'baz', 'foo', and 'qux'")
+        # Create a fake sudo that just echos
+        bin_override = os.path.join(self.path, 'bin')
+        os.mkdir(bin_override)
+
+        fake_sudo_path = os.path.join(bin_override, 'sudo')
+        self.witness_path = os.path.join(self.path, 'sudo_witness')
+        with open(fake_sudo_path, 'w') as f:
+            print('#!/bin/sh', file=f)
+            print('echo $@ >> {}'.format(self.witness_path), file=f)
+        os.chmod(fake_sudo_path, 0o755)
+
+        self.useFixture(fixtures.EnvironmentVariable(
+            'SNAPCRAFT_SETUP_CORE', '1'))
+        self.useFixture(fixtures.EnvironmentVariable(
+            'PATH', '{}:{}'.format(bin_override, os.path.expandvars('$PATH'))))
+
+        self.project_options = snapcraft.ProjectOptions()
+
+    @mock.patch.object(storeapi.StoreClient, 'download')
+    def test_core_setup(self, download_mock):
+        core_snap = self._create_core_snap()
+        core_snap_hash = calculate_sha3_384(core_snap)
+        download_mock.return_value = core_snap_hash
+        self.tempdir_mock.side_effect = self._setup_tempdir_side_effect(
+            core_snap)
+
+        self._create_classic_confined_snapcraft_yaml()
+        lifecycle.execute('pull', self.project_options)
+
+        regex = (
+            'mkdir -p {}\n'
+            'unsquashfs -d {} .*{}\n').format(
+                os.path.dirname(self.core_path),
+                self.core_path,
+                core_snap_hash)
+        self.assertThat(
+            self.witness_path,
+            FileContains(matcher=MatchesRegex(regex, flags=re.DOTALL)))
+
+        download_mock.assert_called_once_with(
+            'core', 'stable', os.path.join(self.tempdir, 'core.snap'),
+            self.project_options.deb_arch, '')
+
+    def test_core_setup_skipped_if_not_classic(self):
+        lifecycle.init()
+        lifecycle.execute('pull', self.project_options)
+
+        self.assertThat(self.witness_path, Not(FileExists()))
+
+    def test_core_setup_skipped_if_core_exists(self):
+        os.makedirs(self.core_path)
+        open(os.path.join(self.core_path, 'fake-content'), 'w').close()
+
+        self._create_classic_confined_snapcraft_yaml()
+        lifecycle.execute('pull', self.project_options)
+
+        self.assertThat(self.witness_path, Not(FileExists()))
+
+    def _create_classic_confined_snapcraft_yaml(self):
+        snapcraft_yaml_path = lifecycle.init()
+        # convert snapcraft.yaml into a classic confined snap
+        with fileinput.FileInput(snapcraft_yaml_path, inplace=True) as f:
+            for line in f:
+                print(line.replace('confinement: devmode',
+                                   'confinement: classic'),
+                      end='')
+
+    def _setup_tempdir_side_effect(self, core_snap):
+        @contextlib.contextmanager
+        def _tempdir():
+            os.makedirs(self.tempdir, exist_ok=True)
+            shutil.move(core_snap, os.path.join(self.tempdir, 'core.snap'))
+            yield self.tempdir
+
+        return _tempdir
+
+    def _create_core_snap(self):
+        core_path = os.path.join(self.path, 'core')
+        snap_yaml_path = os.path.join(core_path, 'meta', 'snap.yaml')
+        os.makedirs(os.path.dirname(snap_yaml_path))
+        with open(snap_yaml_path, 'w') as f:
+            print('name: core', file=f)
+            print('version: 1', file=f)
+            print('architectures: [{}]'.format(
+                self.project_options.deb_arch), file=f)
+            print('summary: summary', file=f)
+            print('description: description', file=f)
+
+        return lifecycle.snap(self.project_options, directory=core_path)
