@@ -1,6 +1,6 @@
 # -*- Mode:Python; indent-tabs-mode:nil; tab-width:4 -*-
 #
-# Copyright (C) 2015, 2016 Canonical Ltd
+# Copyright (C) 2015-2016 Canonical Ltd
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License version 3 as
@@ -14,10 +14,7 @@
 # You should have received a copy of the GNU General Public License
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
-import io
 import logging
-import pkg_resources
-import sys
 from unittest import mock
 
 import fixtures
@@ -53,12 +50,21 @@ class TestMain(TestCase):
         with mock.patch('snapcraft.topic_help') as mock_cmd:
             mock_cmd.side_effect = Exception('some error')
 
-            with self.assertRaises(SystemExit) as raised:
-                snapcraft.main.main(['help', 'topics'])
+            raised = self.assertRaises(
+                SystemExit,
+                snapcraft.main.main, ['help', 'topics'])
 
-        self.assertEqual(1, raised.exception.code)
+        self.assertEqual(1, raised.code)
         self.assertEqual(fake_logger.output, 'some error\n')
 
+    @mock.patch('snapcraft.internal.common.run')
+    def test_command_debug_logs_version(self, mock_run):
+        fake_logger = fixtures.FakeLogger(level=logging.DEBUG)
+        self.useFixture(fake_logger)
+        self.assertRaises(SystemExit, snapcraft.main.main, ['--debug'])
+        self.assertRegex(fake_logger.output,
+                         r'Starting snapcraft \S+ from \S+\.')
+
     @mock.patch('snapcraft.internal.log.configure')
     def test_command_error_debug(self, mock_log_configure):
         with mock.patch('snapcraft.topic_help') as mock_cmd:
@@ -67,10 +73,11 @@ class TestMain(TestCase):
             # When verbose, the exception should be re-raised instead of
             # SystemExit. Note that this test works since SystemExit doesn't
             # inherit from Exception.
-            with self.assertRaises(Exception) as cm:
-                snapcraft.main.main(['help', 'topics', '--debug'])
+            cm = self.assertRaises(
+                Exception,
+                snapcraft.main.main, ['help', 'topics', '--debug'])
 
-        self.assertEqual(str(cm.exception), 'some error')
+        self.assertEqual(str(cm), 'some error')
         mock_log_configure.assert_called_once_with(
             log_level=logging.DEBUG)
 
@@ -105,14 +112,3 @@ class TestMain(TestCase):
             mock_project_options.assert_called_once_with(
                 debug=False, parallel_builds=True, target_deb_arch='arm64',
                 use_geoip=False)
-
-    @mock.patch('pkg_resources.require')
-    @mock.patch('sys.stdout', new_callable=io.StringIO)
-    def test_devel_version(self, mock_stdout, mock_resources):
-        mock_resources.side_effect = pkg_resources.DistributionNotFound()
-        sys.argv = ['/usr/bin/snapcraft', '--version']
-
-        with self.assertRaises(SystemExit):
-            snapcraft.main.main()
-
-        self.assertEqual(mock_stdout.getvalue(), 'devel\n')
