@@ -20,16 +20,19 @@ from unittest import mock
 
 import requests
 import fixtures
+import tempfile
 import yaml
 from collections import OrderedDict
 
 import snapcraft                           # noqa, initialize yaml
-from snapcraft.internal.errors import MissingPackageError
+from snapcraft.internal.errors import (
+    MissingCommandError,
+    SnapcraftEnvironmentError
+)
 from snapcraft.internal import parser
 from snapcraft.internal.parser import (
     _get_origin_data,
     _encode_origin,
-    BadSnapcraftYAMLError,
     PARTS_FILE,
     main,
 )
@@ -59,16 +62,18 @@ def _get_part_list_count(path=PARTS_FILE):
 
 class TestParserBaseDir(TestCase):
     def test__get_base_dir(self):
-        self.assertEqual('/tmp', parser._get_base_dir())
+        self.assertEqual(parser.BASE_DIR, parser._get_base_dir())
 
 
 class TestParser(TestCase):
+
     def tearDown(self):
         try:
             os.remove(PARTS_FILE)
             os.remove(TEST_OUTPUT_PATH)
         except FileNotFoundError:
             pass
+        super().tearDown()
 
     def setUp(self):
         super().setUp()
@@ -80,6 +85,14 @@ class TestParser(TestCase):
         base_dir.return_value = tempdir.path
         self.addCleanup(patcher.stop)
 
+        patcher = mock.patch('snapcraft.internal.repo.check_for_command')
+        self.mock_check_command = patcher.start()
+        self.addCleanup(patcher.stop)
+
+        patcher = mock.patch('snapcraft.internal.sources.get_source_handler')
+        self.mock_get = patcher.start()
+        self.addCleanup(patcher.stop)
+
     def test_ordereddict_yaml(self):
         from collections import OrderedDict
         data = OrderedDict()
@@ -92,8 +105,7 @@ class TestParser(TestCase):
         self.assertTrue(isinstance(yaml.load(output), OrderedDict))
 
     @mock.patch('snapcraft.internal.parser._get_origin_data')
-    @mock.patch('snapcraft.internal.sources.get')
-    def test_main_nested_parts_valid(self, mock_get, mock_get_origin_data):
+    def test_main_nested_parts_valid(self, mock_get_origin_data):
         """Ensure that we fail if there are dependent parts that
         are not included in the wiki's 'parts' section."""
 
@@ -129,8 +141,7 @@ parts: [main, part1, part2]
         self.assertEqual(3, _get_part_list_count())
 
     @mock.patch('snapcraft.internal.parser._get_origin_data')
-    @mock.patch('snapcraft.internal.sources.get')
-    def test_main_nested_parts_invalid(self, mock_get, mock_get_origin_data):
+    def test_main_nested_parts_invalid(self, mock_get_origin_data):
         """Ensure that we fail if there are dependent parts that
         are not included in the wiki's 'parts' section."""
 
@@ -166,8 +177,7 @@ parts: [main, part1]
         self.assertEqual(0, _get_part_list_count())
 
     @mock.patch('snapcraft.internal.parser._get_origin_data')
-    @mock.patch('snapcraft.internal.sources.get')
-    def test_wiki_code_tags(self, mock_get, mock_get_origin_data):
+    def test_wiki_code_tags(self, mock_get_origin_data):
         _create_example_output("""
 {{{
 ---
@@ -189,10 +199,54 @@ parts: [main]
         main(['--debug', '--index', TEST_OUTPUT_PATH])
         self.assertEqual(1, _get_part_list_count())
 
+    @mock.patch('snapcraft.internal.parser._get_base_dir')
     @mock.patch('snapcraft.internal.parser._get_origin_data')
-    @mock.patch('snapcraft.internal.sources.get')
-    def test_main_valid_variable_substition(self, mock_get,
-                                            mock_get_origin_data):
+    def test_origin_options(self, mock_get_origin_data, mock_get_base_dir):
+        _create_example_output("""
+{{{
+---
+maintainer: John Doe <john.doe@example.com
+origin: lp:snapcraft-parser-example
+origin-type: bzr
+origin-branch: stable-branch
+origin-commit: 123
+origin-tag: source-tag
+description: example
+parts: [main]
+}}}
+""")
+        mock_get_base_dir.return_value = tempfile.mkdtemp()
+        mock_get_origin_data.return_value = {
+            'parts': {
+                'main': {
+                    'source': 'lp:something',
+                    'plugin': 'copy',
+                    'files': ['file1', 'file2'],
+                },
+            }
+        }
+        main(['--debug', '--index', TEST_OUTPUT_PATH])
+
+        self.mock_get.assert_has_calls([
+            mock.call('lp:snapcraft-parser-example', source_type='bzr')
+        ])
+
+        mock_source_handler = self.mock_get.return_value
+        mock_source_handler.assert_has_calls([
+            mock.call(
+                'lp:snapcraft-parser-example',
+                source_dir=os.path.join(
+                    parser._get_base_dir(),
+                    _encode_origin('lp:snapcraft-parser-example')))
+        ])
+
+        mock_handler = mock_source_handler.return_value
+        self.assertEqual(mock_handler.source_branch, 'stable-branch')
+        self.assertEqual(mock_handler.source_commit, 123)
+        self.assertEqual(mock_handler.source_tag, 'source-tag')
+
+    @mock.patch('snapcraft.internal.parser._get_origin_data')
+    def test_main_valid_variable_substition(self, mock_get_origin_data):
         _create_example_output("""
 ---
 maintainer: John Doe <john.doe@example.com
@@ -221,8 +275,7 @@ parts: [main]
         self.assertEqual(0, retval)
 
     @mock.patch('snapcraft.internal.parser._get_origin_data')
-    @mock.patch('snapcraft.internal.sources.get')
-    def test_main_valid(self, mock_get, mock_get_origin_data):
+    def test_main_valid(self, mock_get_origin_data):
         _create_example_output("""
 ---
 maintainer: John Doe <john.doe@example.com
@@ -243,9 +296,20 @@ parts: [main]
         self.assertEqual(1, _get_part_list_count())
         self.assertEqual(0, retval)
 
+    def test_404_index(self):
+        retval = main(['--debug', '--index', 'https://fake.example.com'])
+        self.assertIn('Unable to access index: ', self.fake_logger.output)
+        self.assertEqual(1, retval)
+
+    def test_invalid_yaml(self):
+        _create_example_output(':')
+        retval = main(['--debug', '--index', TEST_OUTPUT_PATH])
+        self.assertIn('Bad wiki entry, possibly malformed YAML for entry: ',
+                      self.fake_logger.output)
+        self.assertEqual(1, retval)
+
     @mock.patch('snapcraft.internal.parser._get_origin_data')
-    @mock.patch('snapcraft.internal.sources.get')
-    def test_main_slash_warning(self, mock_get, mock_get_origin_data):
+    def test_main_slash_warning(self, mock_get_origin_data):
         fake_logger = fixtures.FakeLogger(level=logging.WARN)
         self.useFixture(fake_logger)
 
@@ -273,15 +337,22 @@ parts: [main/a]
         self.assertTrue(m in fake_logger.output,
                         'Missing slash deprecation warning in output')
 
-    def test_main_valid_with_default_index(self):
+    def test_main_valid_with_empty_index(self):
+        _create_example_output('')
+        main(['--debug', '--index', TEST_OUTPUT_PATH])
+        self.assertEqual(0, _get_part_list_count())
+
+    @mock.patch('urllib.request.urlopen')
+    def test_main_valid_with_default_index(self, mock_urlopen):
+        class FakeResponse:
+            def read(self):
+                return b''
+        mock_urlopen.return_value = FakeResponse()
         main(['--debug'])
         self.assertEqual(0, _get_part_list_count())
 
     @mock.patch('snapcraft.internal.parser._get_origin_data')
-    @mock.patch('snapcraft.internal.sources.get')
-    def test_main_invalid(self,
-                          mock_get,
-                          mock_get_origin_data):
+    def test_main_invalid(self, mock_get_origin_data):
         _create_example_output("""
 ---
 maintainer: John Doe <john.doe@example.com
@@ -303,10 +374,7 @@ parts: [main, part1]
         self.assertEqual(0, _get_part_list_count())
 
     @mock.patch('snapcraft.internal.parser._get_origin_data')
-    @mock.patch('snapcraft.internal.sources.get')
-    def test_single_part_origin(self,
-                                mock_get,
-                                mock_get_origin_data):
+    def test_single_part_origin(self, mock_get_origin_data):
         """Test a wiki entry with a single origin part."""
         _create_example_output("""
 ---
@@ -329,10 +397,7 @@ parts: [main]
         self.assertEqual(1, _get_part_list_count())
 
     @mock.patch('snapcraft.internal.parser._get_origin_data')
-    @mock.patch('snapcraft.internal.sources.get')
-    def test_multiple_part_origin(self,
-                                  mock_get,
-                                  mock_get_origin_data):
+    def test_multiple_part_origin(self, mock_get_origin_data):
         """Test a wiki entry with multiple origin parts."""
         _create_example_output("""
 ---
@@ -361,10 +426,7 @@ parts: ['main', 'subpart']
         self.assertEqual(2, _get_part_list_count())
 
     @mock.patch('snapcraft.internal.parser._get_origin_data')
-    @mock.patch('snapcraft.internal.sources.get')
-    def test_output_parameter(self,
-                              mock_get,
-                              mock_get_origin_data):
+    def test_output_parameter(self, mock_get_origin_data):
         """Test a wiki entry with multiple origin parts."""
         _create_example_output("""
 ---
@@ -391,10 +453,7 @@ parts: [main]
         self.assertTrue(os.path.exists(filename))
 
     @mock.patch('snapcraft.internal.parser._get_origin_data')
-    @mock.patch('snapcraft.internal.sources.get')
-    def test_source_with_local_part_origin(self,
-                                           mock_get,
-                                           mock_get_origin_data):
+    def test_source_with_local_part_origin(self, mock_get_origin_data):
         """Test a wiki entry with a source with a local part."""
         _create_example_output("""
 ---
@@ -420,10 +479,7 @@ parts: [main]
         self.assertEqual(5, len(part.keys()))
 
     @mock.patch('snapcraft.internal.parser._get_origin_data')
-    @mock.patch('snapcraft.internal.sources.get')
-    def test_source_with_local_subdir_part_origin(self,
-                                                  mock_get,
-                                                  mock_get_origin_data):
+    def test_source_with_local_subdir_part_origin(self, mock_get_origin_data):
         """Test a wiki entry with a source with a local part."""
         _create_example_output("""
 ---
@@ -450,9 +506,8 @@ parts: [main]
         self.assertEqual(6, len(part.keys()))
 
     @mock.patch('snapcraft.internal.parser._get_origin_data')
-    @mock.patch('snapcraft.internal.sources.get')
     def test_source_with_local_source_subdir_part_origin(
-            self, mock_get, mock_get_origin_data):
+            self, mock_get_origin_data):
         """Test a wiki entry with a source with a local source-subdir part."""
         _create_example_output("""
 ---
@@ -480,9 +535,7 @@ parts: [main]
         self.assertEqual(6, len(part.keys()))
 
     @mock.patch('snapcraft.internal.parser._get_origin_data')
-    @mock.patch('snapcraft.internal.sources.get')
-    def test_n_documents(
-            self, mock_get, mock_get_origin_data):
+    def test_n_documents(self, mock_get_origin_data):
         """Test 2 wiki entries."""
         _create_example_output("""
 ---
@@ -515,9 +568,7 @@ parts: [main2]
         self.assertEqual(2, _get_part_list_count())
 
     @mock.patch('snapcraft.internal.parser._get_origin_data')
-    @mock.patch('snapcraft.internal.sources.get')
-    def test_maintaner_is_included(
-            self, mock_get, mock_get_origin_data):
+    def test_maintaner_is_included(self, mock_get_origin_data):
         """Test maintainer is included in parsed parts."""
         _create_example_output("""
 ---
@@ -556,9 +607,7 @@ parts: [main2]
         self.assertEqual(2, _get_part_list_count())
 
     @mock.patch('snapcraft.internal.parser._get_origin_data')
-    @mock.patch('snapcraft.internal.sources.get')
-    def test_description_is_included(
-            self, mock_get, mock_get_origin_data):
+    def test_description_is_included(self, mock_get_origin_data):
         """Test description is included in parsed parts."""
         _create_example_output("""
 ---
@@ -597,8 +646,7 @@ parts: [main2]
         self.assertEqual(2, _get_part_list_count())
 
     @mock.patch('snapcraft.internal.parser._get_origin_data')
-    @mock.patch('snapcraft.internal.sources.get')
-    def test_origin_part_without_source(self, mock_get, mock_get_origin_data):
+    def test_origin_part_without_source(self, mock_get_origin_data):
         _create_example_output("""
 ---
 maintainer: John Doe <john.doe@example.com
@@ -619,8 +667,7 @@ parts: [main]
         self.assertEqual(4, len(_get_part('main').keys()))
 
     @mock.patch('snapcraft.internal.parser._get_origin_data')
-    @mock.patch('snapcraft.internal.sources.get')
-    def test_missing_fields(self, mock_get, mock_get_origin_data):
+    def test_missing_fields(self, mock_get_origin_data):
         _create_example_output("""
 ---
 maintainer: John Doe <john.doe@example.com
@@ -646,10 +693,7 @@ parts: [main]
         self.assertEqual(2, retval)
 
     @mock.patch('snapcraft.internal.parser._get_origin_data')
-    @mock.patch('snapcraft.internal.sources.get')
-    def test_partial_processing_for_malformed_yaml(self,
-                                                   mock_get,
-                                                   mock_get_origin_data):
+    def test_partial_processing_for_malformed_yaml(self, mock_get_origin_data):
         _create_example_output("""
 ---
 maintainer: John Doe <john.doe@example.com>
@@ -688,10 +732,7 @@ parts: [main2]
         self.assertEqual(1, _get_part_list_count())
 
     @mock.patch('snapcraft.internal.parser._get_origin_data')
-    @mock.patch('snapcraft.internal.sources.get')
-    def test_descriptions_get_block_quotes(self,
-                                           mock_get,
-                                           mock_get_origin_data):
+    def test_descriptions_get_block_quotes(self, mock_get_origin_data):
         output = """
 ---
 maintainer: John Doe <john.doe@example.com>
@@ -721,10 +762,7 @@ parts: [main]
         self.assertEqual(1, _get_part_list_count())
 
     @mock.patch('snapcraft.internal.parser._get_origin_data')
-    @mock.patch('snapcraft.internal.sources.get')
-    def test_wiki_interactions_with_fake(self,
-                                         mock_get,
-                                         mock_get_origin_data):
+    def test_wiki_interactions_with_fake(self, mock_get_origin_data):
 
         fixture = fixture_setup.FakePartsWiki()
         self.useFixture(fixture)
@@ -742,10 +780,8 @@ parts: [main]
         self.assertEqual(1, _get_part_list_count())
 
     @mock.patch('snapcraft.internal.parser._get_origin_data')
-    @mock.patch('snapcraft.internal.sources.get')
-    def test_wiki_interactions_with_fake_with_slashes(self,
-                                                      mock_get,
-                                                      mock_get_origin_data):
+    def test_wiki_interactions_with_fake_with_slashes(
+            self, mock_get_origin_data):
 
         fixture = fixture_setup.FakePartsWikiWithSlashes()
         self.useFixture(fixture)
@@ -774,8 +810,7 @@ parts: [main]
         part2 = _get_part('curl-a')
         self.assertTrue(part2)
 
-    @mock.patch('snapcraft.internal.sources.get')
-    def test_missing_snapcraft_yaml(self, mock_get):
+    def test_missing_snapcraft_yaml(self):
 
         fixture = fixture_setup.FakePartsWikiOrigin()
         self.useFixture(fixture)
@@ -796,11 +831,10 @@ parts: [somepart]
         self.assertEqual(0, _get_part_list_count())
 
         self.assertTrue(
-            'Invalid wiki entry'
+            '1 wiki errors found'
             in fake_logger.output, 'Missing invalid wiki entry info in output')
 
-    @mock.patch('snapcraft.internal.sources.get')
-    def test_missing_snapcraft_yaml_without_debug(self, mock_get):
+    def test_missing_snapcraft_yaml_without_debug(self):
 
         fixture = fixture_setup.FakePartsWikiOrigin()
         self.useFixture(fixture)
@@ -817,8 +851,7 @@ parts: [somepart]
         main(['--index', TEST_OUTPUT_PATH])
         self.assertEqual(0, _get_part_list_count())
 
-    @mock.patch('snapcraft.internal.sources.get')
-    def test_wiki_with_fake_origin_with_bad_snapcraft_yaml(self, mock_get):
+    def test_wiki_with_fake_origin_with_bad_snapcraft_yaml(self):
 
         fixture = fixture_setup.FakePartsWikiOrigin()
         self.useFixture(fixture)
@@ -851,8 +884,7 @@ parts: [somepart]
             'Invalid wiki entry'
             in fake_logger.output, 'Missing invalid wiki entry info in output')
 
-    @mock.patch('snapcraft.internal.sources.get')
-    def test_wiki_with_fake_origin(self, mock_get):
+    def test_wiki_with_fake_origin(self):
 
         fixture = fixture_setup.FakePartsWikiOrigin()
         self.useFixture(fixture)
@@ -882,8 +914,7 @@ parts: [somepart]
         self.assertTrue(part)
 
     @mock.patch('snapcraft.internal.parser._get_origin_data')
-    @mock.patch('snapcraft.internal.sources.get')
-    def test_carriage_returns(self, mock_get, mock_get_origin_data):
+    def test_carriage_returns(self, mock_get_origin_data):
         """Test carriage returns in the wiki."""
 
         fake_logger = fixtures.FakeLogger(level=logging.ERROR)
@@ -917,52 +948,8 @@ parts: [main]
 
         self.assertEqual(1, _get_part_list_count())
 
-    @mock.patch('snapcraft.internal.sources.Local.pull')
-    @mock.patch('snapcraft.internal.sources._get_source_type_from_uri')
-    def test_filenotfound_for_non_repos(self, mock_type, mock_pull):
-        mock_pull.side_effect = FileNotFoundError()
-        mock_type.return_value = None
-        fake_logger = fixtures.FakeLogger(level=logging.ERROR)
-        self.useFixture(fake_logger)
-
-        _create_example_output("""
----
-maintainer: John Doe <john.doe@example.com>
-origin: lp:not-a-real-snapcraft-parser-example
-description: example main
-parts: [main]
-""")
-        with self.assertRaises(FileNotFoundError):
-            main(['--debug', '--index', TEST_OUTPUT_PATH])
-
-    @mock.patch('snapcraft.internal.sources.Bazaar.__init__')
-    def test_missing_packages(self, mock_init):
-        mock_init.side_effect = MissingPackageError('bzr')
-        fake_logger = fixtures.FakeLogger(level=logging.ERROR)
-        self.useFixture(fake_logger)
-
-        _create_example_output("""
----
-maintainer: John Doe <john.doe@example.com>
-origin: lp:not-a-real-snapcraft-parser-example
-description: example main
-parts: [main]
----
-maintainer: John Doe <john.doe@example.com>
-origin: lp:not-a-real-snapcraft-parser-example
-description: example main
-parts: [main2]
-""")
-        retval = main(['--debug', '--index', TEST_OUTPUT_PATH])
-        self.assertEqual(2, retval)
-
-        self.assertTrue(
-            'One or more required packages are missing, please install'
-            in fake_logger.output, 'No missing package info in output')
-
     @mock.patch('snapcraft.internal.parser._get_origin_data')
-    @mock.patch('snapcraft.internal.sources.get')
-    def test_duplicate_entries(self, mock_get, mock_get_origin_data):
+    def test_duplicate_entries(self, mock_get_origin_data):
         """Test duplicate parts are ignored."""
 
         fake_logger = fixtures.FakeLogger(level=logging.ERROR)
@@ -1001,9 +988,7 @@ parts: [main]
             in fake_logger.output, 'Missing duplicate part info in output')
 
     @mock.patch('snapcraft.internal.parser._get_origin_data')
-    @mock.patch('snapcraft.internal.sources.get')
-    def test_parsed_output_matches_wiki_order(
-            self, mock_get, mock_get_origin_data):
+    def test_parsed_output_matches_wiki_order(self, mock_get_origin_data):
         _create_example_output("""
 ---
 maintainer: John Doe <john.doe@example.com>
@@ -1056,6 +1041,102 @@ parts: [app1]
         self.assertEqual(parts,
                          _get_part_list())
 
+    @mock.patch('snapcraft.internal.parser._get_origin_data')
+    def test_remote_after_parts(self, mock_get_origin_data):
+        _create_example_output("""
+---
+maintainer: John Doe <john.doe@example.com>
+origin: lp:snapcraft-parser-example
+description: example part on which parent depends on
+parts: [child]
+---
+maintainer: Marco Trevisan <marco@ubuntu.com>
+origin: lp:snapcraft-parser-example
+description: parent part that depends on child
+parts: [parent]
+""")
+        parts = OrderedDict()
+
+        child_part = OrderedDict()
+        child_part['description'] = 'parent part that depends on child'
+        child_part['maintainer'] = 'John Doe <john.doe@example.com>'
+        child_part['plugin'] = 'dump'
+        child_part['source'] = 'lp:project'
+        parts['child'] = child_part
+
+        parent_part = OrderedDict()
+        parent_part['description'] = 'example part on which parent depends on'
+        parent_part['maintainer'] = 'Marco Trevisan <marco@ubuntu.com>'
+        parent_part['plugin'] = 'dump'
+        parent_part['source'] = 'lp:project'
+        parent_part['after'] = ['child']
+        parts['parent'] = parent_part
+
+        mock_get_origin_data.return_value = {
+            'parts': parts,
+        }
+        main(['--index', TEST_OUTPUT_PATH])
+        self.assertEqual(2, _get_part_list_count())
+
+    @mock.patch('snapcraft.internal.parser._get_origin_data')
+    def test_part_missing_in_origin(self, mock_get_origin_data):
+        _create_example_output("""
+---
+maintainer: John Doe <john.doe@example.com>
+origin: lp:snapcraft-parser-example
+description: example part on which parent depends on
+parts: [not-there]
+""")
+
+        mock_get_origin_data.return_value = {
+            'parts': {
+                'there': {
+                    'source': 'lp:project',
+                    'plugin': 'dump',
+                },
+            }
+        }
+        retcode = main(['--index', TEST_OUTPUT_PATH])
+        self.assertEqual(0, _get_part_list_count())
+        self.assertEqual(1, retcode)
+
+    @mock.patch('snapcraft.internal.parser._get_origin_data')
+    def test_remote_after_parts_unordered(self, mock_get_origin_data):
+        _create_example_output("""
+---
+maintainer: Marco Trevisan <marco@ubuntu.com>
+origin: lp:snapcraft-parser-example
+description: parent part that depends on child
+parts: [parent]
+---
+maintainer: John Doe <john.doe@example.com>
+origin: lp:snapcraft-parser-example
+description: example part on which parent depends on
+parts: [child]
+""")
+        parts = OrderedDict()
+
+        parent_part = OrderedDict()
+        parent_part['description'] = 'example part on which parent depends on'
+        parent_part['maintainer'] = 'Marco Trevisan <marco@ubuntu.com>'
+        parent_part['plugin'] = 'dump'
+        parent_part['source'] = 'lp:project'
+        parent_part['after'] = ['child']
+        parts['parent'] = parent_part
+
+        child_part = OrderedDict()
+        child_part['description'] = 'parent part that depends on child'
+        child_part['maintainer'] = 'John Doe <john.doe@example.com>'
+        child_part['plugin'] = 'dump'
+        child_part['source'] = 'lp:project'
+        parts['child'] = child_part
+
+        mock_get_origin_data.return_value = {
+            'parts': parts,
+        }
+        main(['--index', TEST_OUTPUT_PATH])
+        self.assertEqual(2, _get_part_list_count())
+
     def test__get_origin_data_both(self):
         with open(os.path.join(self.tempdir_path,
                   '.snapcraft.yaml'), 'w') as fp:
@@ -1064,7 +1145,8 @@ parts: [app1]
                   'snapcraft.yaml'), 'w') as fp:
             fp.write("")
 
-        self.assertRaises(BadSnapcraftYAMLError, _get_origin_data,
+        self.assertRaises(SnapcraftEnvironmentError,
+                          _get_origin_data,
                           self.tempdir_path)
 
     def test__get_origin_data_hidden_only(self):
@@ -1092,3 +1174,56 @@ parts: [app1]
         origin_dir = _encode_origin(origin)
 
         self.assertEqual('lptestusertestprojecttestbranch', origin_dir)
+
+
+class MissingAssetsTestCase(TestCase):
+
+    def setUp(self):
+        super().setUp()
+
+        patcher = mock.patch('snapcraft.internal.repo.check_for_command')
+        self.mock_check_command = patcher.start()
+        self.addCleanup(patcher.stop)
+
+    @mock.patch('snapcraft.internal.sources.Local.pull')
+    @mock.patch('snapcraft.internal.sources._get_source_type_from_uri')
+    def test_filenotfound_for_non_repos(self, mock_type, mock_pull):
+        mock_pull.side_effect = FileNotFoundError()
+        mock_type.return_value = None
+        fake_logger = fixtures.FakeLogger(level=logging.ERROR)
+        self.useFixture(fake_logger)
+
+        _create_example_output("""
+---
+maintainer: John Doe <john.doe@example.com>
+origin: lp:not-a-real-snapcraft-parser-example
+description: example main
+parts: [main]
+""")
+        self.assertRaises(
+            FileNotFoundError,
+            main, ['--debug', '--index', TEST_OUTPUT_PATH])
+
+    def test_missing_packages(self):
+        self.mock_check_command.side_effect = MissingCommandError('bzr')
+        fake_logger = fixtures.FakeLogger(level=logging.ERROR)
+        self.useFixture(fake_logger)
+
+        _create_example_output("""
+---
+maintainer: John Doe <john.doe@example.com>
+origin: lp:not-a-real-snapcraft-parser-example
+description: example main
+parts: [main]
+---
+maintainer: John Doe <john.doe@example.com>
+origin: lp:not-a-real-snapcraft-parser-example
+description: example main
+parts: [main2]
+""")
+        retval = main(['--debug', '--index', TEST_OUTPUT_PATH])
+        self.assertEqual(2, retval)
+
+        self.assertTrue(
+            'One or more required commands are missing, please install'
+            in fake_logger.output, 'No missing package info in output')
