@@ -1,6 +1,6 @@
 # -*- Mode:Python; indent-tabs-mode:nil; tab-width:4 -*-
 #
-# Copyright (C) 2015, 2016 Canonical Ltd
+# Copyright (C) 2015-2017 Canonical Ltd
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License version 3 as
@@ -23,16 +23,6 @@ class TestSnapcraftExamples(
         testscenarios.WithScenarios, snaps_tests.SnapsTestCase):
 
     scenarios = [
-        ('godd', {
-            'snap_content_dir': 'godd',
-            'name': 'godd',
-            'version': '1.0',
-            }),
-        ('git', {
-            'snap_content_dir': 'git',
-            'name': 'git',
-            'version': '2.8.0',
-            }),
         ('py2-project', {
             'snap_content_dir': 'py2-project',
             'name': 'spongeshaker',
